@@ -4,6 +4,7 @@
 #include <idna.h>
 #include "asynchostresolver.h"
 #include "directorycache.h"
+#include "servercapabilities.h"
 
 DECLARE_EVENT_TYPE(fzOBTAINLOCK, -1)
 DEFINE_EVENT_TYPE(fzOBTAINLOCK)
@@ -101,7 +102,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			nErrorCode == FZ_REPLY_ERROR ||
 			nErrorCode == FZ_REPLY_CRITICALERROR)
 		{
-			return SendNextCommand(nErrorCode);
+			return ParseSubcommandResult(nErrorCode);
 		}
 		else
 			return ResetOperation(nErrorCode);
@@ -195,7 +196,8 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 }
 
 wxString CControlSocket::ConvertDomainName(wxString domain)
-{
+{return domain;
+/*
 	const wxWCharBuffer buffer = wxConvCurrent->cWX2WC(domain);
 
 	int len = 0;
@@ -217,7 +219,7 @@ wxString CControlSocket::ConvertDomainName(wxString domain)
 
 	wxString result = wxConvCurrent->cMB2WX(output);
 	free(output);
-	return result;
+	return result;*/
 }
 
 void CControlSocket::Cancel()
@@ -378,7 +380,12 @@ int CControlSocket::CheckOverwriteFile()
 	bool dirDidExist;
 	bool matchedCase;
 	CDirectoryCache cache;
-	bool found = cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dirDidExist, matchedCase);
+	CServerPath remotePath;
+	if (pData->tryAbsolutePath || m_CurrentPath.IsEmpty())
+		remotePath = pData->remotePath;
+	else
+		remotePath = m_CurrentPath;
+	bool found = cache.LookupFile(entry, *m_pCurrentServer, remotePath, pData->remoteFile, dirDidExist, matchedCase);
 
 	// Ignore entries with wrong case
 	if (found && !matchedCase)
@@ -543,9 +550,9 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
 	return res;
 }
 
-wxCharBuffer CControlSocket::ConvToServer(const wxString& str)
+wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /*=false*/)
 {
-	if (m_useUTF8)
+	if (m_useUTF8 || force_utf8)
 	{
 #if wxUSE_UNICODE
 		wxCharBuffer buffer = wxConvUTF8.cWX2MB(str);
@@ -556,7 +563,7 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str)
 			 buffer = wxConvUTF8.cWC2MB(unicode);
 #endif
 
-		if (buffer)
+		if (buffer || force_utf8)
 			return buffer;
 	}
 
@@ -618,9 +625,15 @@ void CControlSocket::SetWait(bool wait)
 		m_timer.Stop();
 }
 
-int CControlSocket::SendNextCommand(int prevResult /*=FZ_REPLY_OK*/)
+int CControlSocket::SendNextCommand()
 {
-	ResetOperation(prevResult);
+	ResetOperation(FZ_REPLY_INTERNALERROR);
+	return FZ_REPLY_ERROR;
+}
+
+int CControlSocket::ParseSubcommandResult(int prevResult)
+{
+	ResetOperation(FZ_REPLY_INTERNALERROR);
 	return FZ_REPLY_ERROR;
 }
 
@@ -755,6 +768,18 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 	}
 }
 
+wxTimeSpan CControlSocket::GetTimezoneOffset()
+{
+	if (!m_pCurrentServer)
+		return wxTimeSpan();
+
+	int seconds = 0;
+	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) != yes)
+		return wxTimeSpan();
+	
+	return wxTimeSpan(0, 0, seconds);
+}
+
 // ------------------
 // CRealControlSocket
 // ------------------
@@ -923,7 +948,13 @@ int CRealControlSocket::Connect(const CServer &server)
 		delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
-	const wxString & host = server.GetHost();
+	CConnectOpData* pData;
+	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
+		pData = 0;
+	else
+		pData = static_cast<CConnectOpData *>(m_pCurOpData);
+	
+	const wxString& host = pData ? pData->host : server.GetHost();
 	if (!IsIpAddress(host))
 	{
 		LogMessage(Status, _("Resolving IP-Address for %s"), host.c_str());
@@ -959,7 +990,13 @@ int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
 		return DoClose(FZ_REPLY_ERROR | FZ_REPLY_CRITICALERROR);
 	}
 
-	const unsigned int port = m_pCurrentServer->GetPort();
+	CConnectOpData* pData;
+	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
+		pData = 0;
+	else
+		pData = static_cast<CConnectOpData *>(m_pCurOpData);
+
+	const unsigned int port = pData ? pData->port : m_pCurrentServer->GetPort();
 	LogMessage(Status, _("Connecting to %s:%d..."), address->IPAddress().c_str(), port);
 
 	wxIPV4address addr = *address;
