@@ -12,7 +12,6 @@ std::map<wxString, int> CDirectoryListingParser::m_MonthNamesMap;
 //#define LISTDEBUG
 #ifdef LISTDEBUG
 static char data[][110]={
-
 	/* IBM MVS listings */
 	// Volume Unit    Referred Ext Used Recfm Lrecl BlkSz Dsorg Dsname
 	"  WYOSPT 3420   2003/05/21  1  200  FB      80  8053  PS  48-MVS.FILE",
@@ -32,6 +31,25 @@ static char data[][110]={
 	// Migrated MVS file
 	"Migrated				SOME.FILE",
 
+	// z/VM, another IBM abomination. Description by Alexandre Charbey
+	// 
+	// ACTAPI
+	//   is a filename
+	// TRACE
+	//   is a filetype (extension, like exe or com or jpg...)
+	// V
+	//   is the file format. Designates how records are arranged in a file. F=Fixed and V=Variable. I don't think you care
+	// 65
+	//   is the logical record length.
+	// 107
+	//   is Number of records in a file.
+	// 2
+	//   (seems wrong) is the block size ( iirc 1 is 127, 2 is 381, 3 is 1028 and 4 is 4072 - not sure - the numbers are not the usual binary numbers)
+	// there is the date/time
+	// 060191
+	//   I think it is some internal stuff saying who the file belongs to.  191 is the "handle" of the user's disk. I don't know what 060 is. This 060191 is what FZ shows in its file list.
+	"ACTAPI  TRACE   V        65      107        2 2005-10-04 15:28:42 060191",
+
 	""};
 
 #endif
@@ -594,7 +612,7 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("avg")] = 8;
 
 		//There are more languages and thus month
-		//names, but as long as knowbody reports a
+		//names, but as long as nobody reports a
 		//problem, I won't add them, there are way
 		//too many languages
 
@@ -726,7 +744,22 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 bool CDirectoryListingParser::ParseLine(CLine *pLine, const enum ServerType serverType, bool concatenated)
 {
 	CDirentry entry;
-	bool res = ParseAsUnix(pLine, entry);
+	bool res;
+
+	if (serverType == ZVM)
+	{
+		res = ParseAsZVM(pLine, entry);
+		if (res)
+			goto done;
+	}
+	else if (serverType == HPNONSTOP)
+	{
+		res = ParseAsHPNonstop(pLine, entry);
+		if (res)
+			goto done;
+	}
+
+	res = ParseAsUnix(pLine, entry);
 	if (res)
 		goto done;
 	res = ParseAsDos(pLine, entry);
@@ -941,6 +974,9 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry)
 			}
 		}
 
+		if (entry.hasTime)
+			entry.time.Add(m_timezoneOffset);
+
 		return true;
 	}
 	while (numOwnerGroup--);
@@ -1085,7 +1121,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 			strMonth = strMonth.Left(i + 1);
 		}
 		// Check month name
-		if (strMonth.Right(1) == _T(",") || strMonth.Right(1) == _T("."))
+		while (strMonth.Right(1) == _T(",") || strMonth.Right(1) == _T("."))
 			strMonth.RemoveLast();
 		strMonth.MakeLower();
 		std::map<wxString, int>::iterator iter = m_MonthNamesMap.find(strMonth);
@@ -1123,8 +1159,15 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 		{
 			year = wxDateTime::Now().GetYear();
 			int currentDayOfYear = wxDateTime::Now().GetDay() + 31 * (wxDateTime::Now().GetMonth() - wxDateTime::Jan);
-			int fileDayOfYear = (month - 1) * 31 + (day - 1);
-			if (currentDayOfYear <= fileDayOfYear)
+			int fileDayOfYear = (month - 1) * 31 + day;
+
+			// We have to compare with an offset of one. In the worst case,
+			// the server's timezone might be up to 24 hours ahead of the 
+			// client.
+			// Problem: Servers which do send the time but not the year even
+			// one day away from getting 1 year old. This is far more uncommon
+			// however.
+			if ((currentDayOfYear + 1) < fileDayOfYear)
 				year -= 1;
 		}
 	}
@@ -1238,7 +1281,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		if (token[pos] == '.')
 		{
 			// Maybe dd.mm.yyyy
-			if (value < 1900 || value > 3000)
+			if (value < 1 || value > 31)
 				return false;
 			day = value.GetLo();
 			gotDay = true;
@@ -1419,6 +1462,9 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
 	return true;
 }
 
@@ -1435,10 +1481,32 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 	if (hour < 0 || hour > 23)
 		return false;
 
-	wxLongLong minute = token.GetNumber(pos + 1, -1);
+	// See if we got seconds
+	int pos2 = token.Find(':', pos + 1);
+	int len;
+	if (pos2 == -1)
+		len = -1;
+	else
+		len = pos2 - pos - 1;
+
+	if (!len)
+		return false;
+
+	wxLongLong minute = token.GetNumber(pos + 1, len);
 	if (minute < 0 || minute > 59)
 		return false;
 
+	wxLongLong seconds;
+	if (pos2 == -1)
+		seconds = 0;
+	else
+	{
+		// Parse seconds
+		seconds = token.GetNumber(pos2 + 1, -1);
+		if (seconds < 0 || seconds > 59)
+			return false;
+	}
+
 	// Convert to 24h format
 	if (!token.IsRightNumeric())
 	{
@@ -1452,7 +1520,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 				hour = 0;
 	}
 
-	wxTimeSpan span(hour.GetLo(), minute.GetLo());
+	wxTimeSpan span(hour.GetLo(), minute.GetLo(), seconds.GetLo());
 	entry.time.Add(span);
 	entry.hasTime = true;
 
@@ -1523,6 +1591,24 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 	return true;
 }
 
+wxString Unescape(const wxString& str, wxChar escape)
+{
+	wxString res;
+	for (unsigned int i = 0; i < str.Len(); i++)
+	{
+		wxChar c = str[i];
+		if (c == escape)
+		{
+			c = str[++i];
+			if (!c)
+				break;
+		}
+		res += c;
+	}
+
+	return res;
+}
+
 bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 {
 	CToken token;
@@ -1538,7 +1624,10 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	if (pos > 4 && token.GetString().Mid(pos - 4, 4) == _T(".DIR"))
 	{
 		entry.dir = true;
-		entry.name = token.GetString().Left(pos - 4) + token.GetString().Mid(pos);
+		if (token.GetString().Mid(pos) == _(";1"))
+			entry.name = token.GetString().Left(pos - 4);
+		else
+			entry.name = token.GetString().Left(pos - 4) + token.GetString().Mid(pos);
 	}
 	else
 	{
@@ -1546,12 +1635,14 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		entry.name = token.GetString();
 	}
 
+	// Some VMS servers escape special characters like additional dots with ^
+	entry.name = Unescape(entry.name, '^');
+
 	if (!pLine->GetToken(++index, token))
 		return false;
 
 	entry.ownerGroup = _T("");
 
-	bool gotSize = false;
 	// This field can either be the filesize, a username (at least that's what I think) enclosed in [] or a date.
 	if (!token.IsNumeric() && !token.IsLeftNumeric())
 	{
@@ -1569,16 +1660,44 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	}
 
 	// Current token is either size or date
+	bool gotSize = false;
 	pos = token.Find('/');
+	
+	if (!pos)
+		return false;
+
 	if (token.IsNumeric() || (pos != -1 && token.Find('/', pos + 1) == -1))
 	{
 		// Definitely size
+
+		CToken sizeToken;
+		if (pos == -1)
+			sizeToken = token;
+		else
+			sizeToken = CToken(token.GetToken(), pos);
+		if (!ParseComplexFileSize(sizeToken, entry.size, 512))
+			return false;
 		gotSize = true;
-		entry.size = token.GetNumber();
 
 		if (!pLine->GetToken(++index, token))
 			return false;
 	}
+	else if (pos == -1 && token.IsLeftNumeric())
+	{
+		// Perhaps size
+		CToken sizeToken;
+		if (pos == -1)
+			sizeToken = token;
+		else
+			sizeToken = CToken(token.GetToken(), pos);
+		if (ParseComplexFileSize(sizeToken, entry.size, 512))
+		{
+			gotSize = true;
+
+			if (!pLine->GetToken(++index, token))
+				return false;
+		}
+	}
 
 	// Get date
 	if (!ParseShortDate(token, entry))
@@ -1612,7 +1731,17 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		if (!token.IsNumeric() && !token.IsLeftNumeric())
 			return false;
 
-		entry.size = token.GetNumber();
+		int pos = token.Find('/');
+		if (!pos)
+			return false;
+
+		CToken sizeToken;
+		if (pos == -1)
+			sizeToken = token;
+		else
+			sizeToken = CToken(token.GetToken(), pos);
+		if (!ParseComplexFileSize(sizeToken, entry.size, 512))
+			return false;
 	}
 
 	// Owner / group and permissions
@@ -1640,6 +1769,9 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		}
 	}
 
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
 	return true;
 }
 
@@ -1690,6 +1822,9 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 	else
 		entry.dir = false;
 
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
 	return true;
 }
 
@@ -1870,6 +2005,9 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 		entry.target = _T("");
 		entry.ownerGroup = _T("");
 		entry.permissions = _T("");
+
+		if (entry.hasTime)
+			entry.time.Add(m_timezoneOffset);
 	}
 
 	return true;
@@ -2091,6 +2229,9 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine *pLine, CDirentry &entry)
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
 	return true;
 }
 
@@ -2262,9 +2403,11 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry)
 	entry.dir = false;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
-	entry.hasTime = false;
 	entry.link = false;
 
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
 	return true;
 }
 
@@ -2365,25 +2508,40 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry
 	return true;
 }
 
-bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, wxLongLong& size)
+bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, wxLongLong& size, int blocksize /*=-1*/)
 {
 	if (token.IsNumeric())
 	{
 		size = token.GetNumber();
+		if (blocksize != -1)
+			size *= blocksize;
+
 		return true;
 	}
 
-	int len = token.GetLength() - 1;
+	int len = token.GetLength();
 
-	char last = token[len];
+	char last = token[len - 1];
 	if (last == 'B' || last == 'b')
 	{
-		char c = token[len];
+		if (len == 1)
+			return false;
+
+		char c = token[--len - 1];
 		if (c < '0' || c > '9')
 		{
-			last = token[len];
 			len--;
+			last = c;
 		}
+		else
+			last = 0;
+	}
+	else if (last >= '0' && last <= '9')
+		last = 0;
+	else
+	{
+		if (--len == 0)
+			return false;
 	}
 
 	size = 0;
@@ -2428,6 +2586,10 @@ bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, wxLongLong& si
 	case 'b':
 	case 'B':
 		break;
+	case 0:
+		if (blocksize != -1)
+			size *= blocksize;
+		break;
 	default:
 		return false;
 	}
@@ -2524,8 +2686,6 @@ bool CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.unsure = false;
-
 	return true;
 }
 
@@ -2591,7 +2751,6 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 	entry.hasTime = false;
 	entry.link = false;
-	entry.unsure = false;
 
 	return true;
 }
@@ -2611,3 +2770,151 @@ void CDirectoryListingParser::Reset()
 	m_fileListOnly = true;
 	m_maybeMultilineVms = false;
 }
+
+bool CDirectoryListingParser::ParseAsZVM(CLine* pLine, CDirentry &entry)
+{
+	int index = 0;
+	CToken token;
+
+	// Get name
+	if (!pLine->GetToken(index, token))
+		return false;
+
+	entry.name = token.GetString();
+
+	// Get filename extension
+	if (!pLine->GetToken(++index, token))
+		return false;
+	entry.name += _T(".") + token.GetString();
+
+	// File format. Unused
+	if (!pLine->GetToken(++index, token))
+		return false;
+	wxString format = token.GetString();
+	if (format != _T("V") && format != _T("F"))
+		return false;
+
+	// Record length
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	if (!token.IsNumeric())
+		return false;
+
+	entry.size = token.GetNumber();
+
+	// Number of records
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	if (!token.IsNumeric())
+		return false;
+
+	entry.size *= token.GetNumber();
+
+	// Unused (Block size?)
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	if (!token.IsNumeric())
+		return false;
+
+	// Date
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	if (!ParseShortDate(token, entry, true))
+		return false;
+
+	// Time
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	if (!ParseTime(token, entry))
+		return false;
+
+	// Owner
+	if (!pLine->GetToken(++index, token))
+		return false;
+
+	entry.ownerGroup = token.GetString();
+
+	// No further token!
+	if (pLine->GetToken(++index, token))
+		return false;
+
+	entry.dir = false;
+	entry.link = false;
+	entry.permissions = _T("");
+	entry.target = _T("");
+
+	if (entry.hasTime)
+		entry.time.Add(m_timezoneOffset);
+
+	return true;
+}
+
+bool CDirectoryListingParser::ParseAsHPNonstop(CLine *pLine, CDirentry &entry)
+{
+	int index = 0;
+	CToken token;
+
+	// Get name
+	if (!pLine->GetToken(index, token))
+		return false;
+
+	entry.name = token.GetString();
+
+	// File code, numeric, unsuded
+	if (!pLine->GetToken(++index, token))
+		return false;
+	if (!token.IsNumeric())
+		return false;
+
+	// Size
+	if (!pLine->GetToken(++index, token))
+		return false;
+	if (!token.IsNumeric())
+		return false;
+
+	entry.size = token.GetNumber();
+
+	// Date
+	if (!pLine->GetToken(++index, token))
+		return false;
+	if (!ParseShortDate(token, entry, false))
+		return false;
+
+	// Time
+	if (!pLine->GetToken(++index, token))
+		return false;
+	if (!ParseTime(token, entry))
+		return false;
+
+	// Owner
+	if (!pLine->GetToken(++index, token))
+		return false;
+	entry.ownerGroup = token.GetString();
+
+	if (token[token.GetLength() - 1] == ',')
+	{
+		// Owner, part 2
+		if (!pLine->GetToken(++index, token))
+			return false;
+		entry.ownerGroup += _T(" ") + token.GetString();
+	}
+
+	// Permissions
+	if (!pLine->GetToken(++index, token))
+		return false;
+	entry.permissions = token.GetString();
+
+	// Nothing
+	if (pLine->GetToken(++index, token))
+		return false;
+
+	entry.dir = false;
+	entry.link = false;
+		
+	return true;
+}
