@@ -228,12 +228,16 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 				const CConnectCommand *pConnectCommand = (CConnectCommand *)m_pCurrentCommand;
 
 				RegisterFailedLoginAttempt(pConnectCommand->GetServer());
-				unsigned int delay = GetRemainingReconnectDelay(pConnectCommand->GetServer());
-				if (!delay)
-					delay = 1;
-				m_pLogging->LogMessage(Status, _("Waiting to retry..."));
-				m_retryTimer.Start(delay, true);
-				return FZ_REPLY_WOULDBLOCK;
+
+				if (pConnectCommand->RetryConnecting())
+				{
+					unsigned int delay = GetRemainingReconnectDelay(pConnectCommand->GetServer());
+					if (!delay)
+						delay = 1;
+					m_pLogging->LogMessage(Status, _("Waiting to retry..."));
+					m_retryTimer.Start(delay, true);
+					return FZ_REPLY_WOULDBLOCK;
+				}
 			}
 		}
 
@@ -252,12 +256,13 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 	}
 	else if (nErrorCode & FZ_REPLY_DISCONNECTED)
 	{
-		wxASSERT(!m_bIsInCommand);
-		
-		COperationNotification *notification = new COperationNotification();
-		notification->nReplyCode = nErrorCode;
-		notification->commandId = cmd_none;
-		AddNotification(notification);
+		if (!m_bIsInCommand)
+		{
+			COperationNotification *notification = new COperationNotification();
+			notification->nReplyCode = nErrorCode;
+			notification->commandId = cmd_none;
+			AddNotification(notification);
+		}
 	}
 
 	if (!m_HostResolverThreads.empty())
@@ -271,15 +276,12 @@ void CFileZillaEnginePrivate::SetActive(bool recv)
 	if (m_pControlSocket)
 		m_pControlSocket->SetAlive();
 
-	m_lock.Enter();
 	if (recv)
 	{
 		if (!m_activeStatusRecv)
 		{
 			m_activeStatusRecv = 2;
-			m_lock.Leave();
 			AddNotification(new CActiveNotification(true));
-			return;
 		}
 	}
 	else
@@ -287,12 +289,9 @@ void CFileZillaEnginePrivate::SetActive(bool recv)
 		if (!m_activeStatusSend)
 		{
 			m_activeStatusSend = 2;
-			m_lock.Leave();
 			AddNotification(new CActiveNotification(false));
-			return;
 		}
 	}
-	m_lock.Leave();
 }
 
 unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
@@ -325,7 +324,7 @@ int CFileZillaEnginePrivate::Connect(const CConnectCommand &command)
 
 	if (command.GetServer().GetPort() != CServer::GetDefaultPort(command.GetServer().GetProtocol()))
 	{
-		ServerProtocol protocol = CServer::GetProtocolFromPort(command.GetServer().GetPort());
+		ServerProtocol protocol = CServer::GetProtocolFromPort(command.GetServer().GetPort(), true);
 		if (protocol != UNKNOWN && protocol != command.GetServer().GetProtocol())
 			m_pLogging->LogMessage(Status, _("Selected port usually in use by a different protocol."));
 	}
@@ -393,10 +392,10 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 		{
 			CDirectoryListing *pListing = new CDirectoryListing;
 			CDirectoryCache cache;
-			bool found = cache.Lookup(*pListing, *pServer, command.GetPath(), command.GetSubDir(), true);
-			if (found)
+			bool is_outdated = false;
+			bool found = cache.Lookup(*pListing, *pServer, command.GetPath(), command.GetSubDir(), true, is_outdated);
+			if (found && !is_outdated)
 			{
-				
 				if (pListing->m_hasUnsureEntries)
 					refresh = true;
 				else
@@ -409,6 +408,8 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 					return FZ_REPLY_OK;
 				}
 			}
+			if (is_outdated)
+				refresh = true;
 			delete pListing;
 		}
 	}
@@ -455,11 +456,11 @@ int CFileZillaEnginePrivate::Delete(const CDeleteCommand& command)
 		return FZ_REPLY_BUSY;
 
 	if (command.GetPath().IsEmpty() ||
-		command.GetFile() == _T(""))
+		command.GetFiles().empty())
 		return FZ_REPLY_SYNTAXERROR;
 
 	m_pCurrentCommand = command.Clone();
-	return m_pControlSocket->Delete(command.GetPath(), command.GetFile());
+	return m_pControlSocket->Delete(command.GetPath(), command.GetFiles());
 }
 
 int CFileZillaEnginePrivate::RemoveDir(const CRemoveDirCommand& command)
@@ -639,7 +640,7 @@ void CFileZillaEnginePrivate::OnTimer(wxTimerEvent& event)
 	}
 	wxASSERT(!IsConnected());
 
-#ifdef __WXDEBUG__
+#if 0//__WXDEBUG__
 	const CConnectCommand *pConnectCommand = (CConnectCommand *)m_pCurrentCommand;
 	wxASSERT(!GetRemainingReconnectDelay(pConnectCommand->GetServer()));
 #endif
