@@ -285,42 +285,82 @@ bool CServer::operator<(const CServer &op) const
 {
 	if (m_protocol < op.m_protocol)
 		return true;
-	else if (m_type < op.m_type)
+	else if (m_protocol > op.m_protocol)
+		return false;
+
+	if (m_type < op.m_type)
 		return true;
-	else if (m_host < op.m_host)
+	else if (m_type > op.m_type)
+		return false;
+
+	int cmp = m_host.Cmp(op.m_host);
+	if (cmp < 0)
 		return true;
-	else if (m_port < op.m_port)
+	else if (cmp > 0)
+		return false;
+
+	if (m_port < op.m_port)
 		return true;
-	else if (m_logonType < op.m_logonType)
+	else if (m_port > op.m_port)
+		return false;
+
+	if (m_logonType < op.m_logonType)
 		return true;
-	else if (m_logonType != ANONYMOUS)
+	else if (m_logonType > op.m_logonType)
+		return false;
+
+	if (m_logonType != ANONYMOUS)
 	{
-		if (m_user < op.m_user)
+		cmp = m_user.Cmp(op.m_user);
+		if (cmp < 0)
 			return true;
+		else if (cmp > 0)
+			return false;
 
 		if (m_logonType == NORMAL)
 		{
-			if (m_pass < op.m_pass)
+			cmp = m_pass.Cmp(op.m_pass);
+			if (cmp < 0)
 				return true;
+			else if (cmp > 0)
+				return false;
 		}
-		else if (m_logonType == NORMAL)
+		else if (m_logonType == ACCOUNT)
 		{
-			if (m_pass < op.m_pass)
+			cmp = m_pass.Cmp(op.m_pass);
+			if (cmp < 0)
 				return true;
-			if (m_account < op.m_account)
+			else if (cmp > 0)
+				return false;
+
+			cmp = m_account.Cmp(op.m_account);
+			if (cmp < 0)
 				return true;
+			else if (cmp > 0)
+				return false;
 		}
 	}
 	if (m_timezoneOffset < op.m_timezoneOffset)
 		return true;
-	else if (m_pasvMode < op.m_pasvMode)
+	else if (m_timezoneOffset > op.m_timezoneOffset)
+		return false;
+	
+	if (m_pasvMode < op.m_pasvMode)
 		return true;
-	else if (m_encodingType < op.m_encodingType)
+	else if (m_pasvMode > op.m_pasvMode)
+		return false;
+
+	if (m_encodingType < op.m_encodingType)
 		return true;
-	else if (m_encodingType == ENCODING_CUSTOM)
+	else if (m_encodingType > op.m_encodingType)
+		return false;
+
+	if (m_encodingType == ENCODING_CUSTOM)
 	{
 		if (m_customEncoding < op.m_customEncoding)
 			return true;
+		else if (m_customEncoding > op.m_customEncoding)
+			return false;
 	}
 
 	// Do not compare number of allowed multiple connections
@@ -423,7 +463,7 @@ bool CServer::SetAccount(const wxString& account)
 
 bool CServer::SetTimezoneOffset(int minutes)
 {
-	if (minutes > (60 * 13) || minutes < (-60 * 30))
+	if (minutes > (60 * 24) || minutes < (-60 * 24))
 		return false;
 
 	m_timezoneOffset = minutes;
@@ -552,7 +592,7 @@ unsigned int CServer::GetDefaultPort(enum ServerProtocol protocol)
 	return 21;
 }
 
-enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port)
+enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port, bool defaultOnly /*=false*/)
 {
 	for (unsigned int i = 0; protocolInfos[i].protocol != UNKNOWN; i++)
 	{
@@ -560,7 +600,10 @@ enum ServerProtocol CServer::GetProtocolFromPort(unsigned int port)
 			return protocolInfos[i].protocol;
 	}
 	
-	// Default to FTP
+	if (defaultOnly)
+		return UNKNOWN;
+
+	// Else default to FTP
 	return FTP;
 }
 
