@@ -22,49 +22,18 @@ enum filetransferStates
 	filetransfer_transfer
 };
 
-extern const wxEventType fzEVT_SFTP;
-typedef void (wxEvtHandler::*fzSftpEventFunction)(CSftpEvent&);
-
-#define EVT_SFTP(fn) \
-	DECLARE_EVENT_TABLE_ENTRY( \
-		fzEVT_SFTP, -1, -1, \
-		(wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( fzSftpEventFunction, &fn ), \
-		(wxObject *) NULL \
-	),
-
-DEFINE_EVENT_TYPE(fzEVT_SFTP);
-
-CSftpEvent::CSftpEvent(sftpEventTypes type, const wxString& text)
-	: wxEvent(wxID_ANY, fzEVT_SFTP)
+struct sftp_message
 {
-	m_type = type;
-	m_text[0] = text;
-	m_reqType = sftpReqUnknown;
-}
+	sftpEventTypes type;
+	wxString text;
+	sftpRequestTypes reqType;
+};
 
-CSftpEvent::CSftpEvent(sftpEventTypes type, sftpRequestTypes reqType, const wxString& text1, const wxString& text2 /*=_T("")*/, const wxString& text3 /*=_T("")*/, const wxString& text4 /*=_T("")*/)
-	: wxEvent(wxID_ANY, fzEVT_SFTP)
-{
-	wxASSERT(type == sftpRequest || reqType == sftpReqUnknown);
-	m_type = type;
-
-	m_reqType = reqType;
-
-	m_text[0] = text1;
-	m_text[1] = text2;
-	m_text[2] = text3;
-	m_text[3] = text4;
-}
-
-int CSftpEvent::GetNumber() const
-{
-	long number = 0;
-	m_text[0].ToLong(&number);
-	return number;
-}
+DECLARE_EVENT_TYPE(fzEVT_SFTP, -1)
+DEFINE_EVENT_TYPE(fzEVT_SFTP)
 
 BEGIN_EVENT_TABLE(CSftpControlSocket, CControlSocket)
-EVT_SFTP(CSftpControlSocket::OnSftpEvent)
+EVT_COMMAND(wxID_ANY, fzEVT_SFTP, CSftpControlSocket::OnSftpEvent)
 EVT_END_PROCESS(wxID_ANY, CSftpControlSocket::OnTerminate)
 END_EVENT_TABLE();
 
@@ -75,11 +44,14 @@ public:
 		: wxThreadEx(wxTHREAD_JOINABLE), m_pProcess(pProcess),
 		  m_pOwner(pOwner)
 	{
-
 	}
 
 	virtual ~CSftpInputThread()
 	{
+		m_criticalSection.Enter();
+		for (std::list<sftp_message*>::iterator iter = m_sftpMessages.begin(); iter != m_sftpMessages.end(); iter++)
+			delete *iter;
+		m_criticalSection.Leave();
 	}
 
 	bool Init()
@@ -91,9 +63,30 @@ public:
 
 		return true;
 	}
-	
+
+	void GetMessages(std::list<sftp_message*>& messages)
+	{
+		m_criticalSection.Enter();
+		messages.swap(m_sftpMessages);
+		m_criticalSection.Leave();
+	}
+
 protected:
 
+	void SendMessage(sftp_message* message)
+	{
+		bool sendEvent;
+
+		m_criticalSection.Enter();
+		sendEvent = m_sftpMessages.empty();
+		m_sftpMessages.push_back(message);
+		m_criticalSection.Leave();
+
+		wxCommandEvent evt(fzEVT_SFTP, wxID_ANY);
+		if (sendEvent)
+			wxPostEvent(m_pOwner, evt);
+	}
+
 	wxString ReadLine(wxInputStream* pInputStream, bool &error)
 	{
 		int read = 0;
@@ -122,7 +115,7 @@ protected:
 				// Cap string length
 				continue;
 			}
-			
+
 			buffer[read++] = c;
 		}
 		if (pInputStream->Eof())
@@ -160,52 +153,27 @@ protected:
 				break;
 
 			eventType -= '0';
-			
+
 			switch(eventType)
 			{
 			case sftpReply:
 			case sftpListentry:
 			case sftpRequestPreamble:
 			case sftpRequestInstruction:
-				{
-					const wxString& line = ReadLine(pInputStream, error);
-					if (error)
-						goto loopexit;
-					CSftpEvent evt((sftpEventTypes)eventType, line);
-					wxPostEvent(m_pOwner, evt);
-				}
-				break;
+			case sftpDone:
 			case sftpError:
-				{
-					const wxString& line = ReadLine(pInputStream, error);
-					if (error)
-						goto loopexit;
-					m_pOwner->LogMessageRaw(::Error, line);
-				}
-				break;
 			case sftpVerbose:
-				{
-					const wxString& line = ReadLine(pInputStream, error);
-					if (error)
-						goto loopexit;
-					m_pOwner->LogMessageRaw(Debug_Info, line);
-				}
-				break;
 			case sftpStatus:
 				{
-					const wxString& line = ReadLine(pInputStream, error);
+					sftp_message* message = new sftp_message;
+					message->type = (sftpEventTypes)eventType;
+					message->text = ReadLine(pInputStream, error).c_str();
 					if (error)
+					{
+						delete message;
 						goto loopexit;
-					m_pOwner->LogMessageRaw(Status, line);
-				}
-				break;
-			case sftpDone:
-				{
-					const wxString& line = ReadLine(pInputStream, error);
-					if (error)
-						goto loopexit;
-					CSftpEvent evt((sftpEventTypes)eventType, line);
-					wxPostEvent(m_pOwner, evt);
+					}
+					SendMessage(message);
 				}
 				break;
 			case sftpRequest:
@@ -230,33 +198,36 @@ protected:
 					}
 					else if (requestType == sftpReqPassword)
 					{
-						CSftpEvent evt(sftpRequest, sftpReqPassword, line.Mid(1));
-						wxPostEvent(m_pOwner, evt);
+						sftp_message* message = new sftp_message;
+						message->type = (sftpEventTypes)eventType;
+						message->reqType = sftpReqPassword;
+						message->text = line.Mid(1).c_str();
+						SendMessage(message);
 					}
 				}
 				break;
 			case sftpRecv:
-				m_pOwner->SetActive(true);
-				break;
 			case sftpSend:
-				m_pOwner->SetActive(false);
+			case sftpUsedQuotaRecv:
+			case sftpUsedQuotaSend:
+				{
+					sftp_message* message = new sftp_message;
+					message->type = (sftpEventTypes)eventType;
+					SendMessage(message);
+				}
 				break;
 			case sftpRead:
 			case sftpWrite:
 				{
-					wxTextInputStream textStream(*pInputStream);
-					wxString text = textStream.ReadLine();
-					if (pInputStream->LastRead() <= 0 || text == _T(""))
+					sftp_message* message = new sftp_message;
+					message->type = (sftpEventTypes)eventType;
+					message->text = ReadLine(pInputStream, error);
+					if (pInputStream->LastRead() <= 0 || message->text == _T(""))
+					{
+						delete message;
 						goto loopexit;
-					CSftpEvent evt((sftpEventTypes)eventType, text);
-					wxPostEvent(m_pOwner, evt);
-				}
-				break;
-			case sftpUsedQuotaRecv:
-			case sftpUsedQuotaSend:
-				{
-					CSftpEvent evt((sftpEventTypes)eventType, _T(""));
-					wxPostEvent(m_pOwner, evt);
+					}
+					SendMessage(message);
 				}
 				break;
 			default:
@@ -281,6 +252,34 @@ loopexit:
 
 	wxProcess* m_pProcess;
 	CSftpControlSocket* m_pOwner;
+
+	std::list<sftp_message*> m_sftpMessages;
+	wxCriticalSection m_criticalSection;
+};
+
+class CSftpDeleteOpData : public COpData
+{
+public:
+	CSftpDeleteOpData()
+		: COpData(cmd_delete)
+	{
+		m_needSendListing = false;
+		m_deleteFailed = false;
+	}
+
+	virtual ~CSftpDeleteOpData() { }
+
+	CServerPath path;
+	std::list<wxString> files;
+
+	// Set to wxDateTime::UNow initially and after
+	// sending an updated listing to the UI.
+	wxDateTime m_time;
+
+	bool m_needSendListing;
+
+	// Set to true if deletion of at least one file failed
+	bool m_deleteFailed;
 };
 
 CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate *pEngine) : CControlSocket(pEngine)
@@ -324,6 +323,19 @@ int CSftpControlSocket::Connect(const CServer &server)
 	LogMessage(Status, _("Connecting to %s:%d..."), server.GetHost().c_str(), server.GetPort());
 	SetWait(true);
 
+	delete m_pCSConv;
+	if (server.GetEncodingType() == ENCODING_CUSTOM)
+	{
+		m_pCSConv = new wxCSConv(server.GetCustomEncoding());
+		m_useUTF8 = false;
+	}
+	else
+	{
+		m_pCSConv = 0;
+		m_useUTF8 = true;
+	}
+
+
 	if (m_pCurrentServer)
 		delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
@@ -348,7 +360,7 @@ int CSftpControlSocket::Connect(const CServer &server)
 		DoClose();
 		return FZ_REPLY_ERROR;
 	}
-	
+
 	m_pInputThread = new CSftpInputThread(this, m_pProcess);
 	if (!m_pInputThread->Init())
 	{
@@ -395,7 +407,7 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 	}
 
 	pData->gotInitialReply = true;
-	
+
 	bool res = Send(wxString::Format(_T("open \"%s@%s\" %d"), m_pCurrentServer->GetUser().c_str(), m_pCurrentServer->GetHost().c_str(), m_pCurrentServer->GetPort()));
 
 	if (res)
@@ -404,104 +416,136 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 		return FZ_REPLY_ERROR;
 }
 
-void CSftpControlSocket::OnSftpEvent(CSftpEvent& event)
+void CSftpControlSocket::OnSftpEvent(wxCommandEvent& event)
 {
 	if (!m_pCurrentServer)
 		return;
 
-	switch (event.GetType())
+	if (!m_pInputThread)
+		return;
+
+	std::list<sftp_message*> messages;
+	m_pInputThread->GetMessages(messages);
+	for (std::list<sftp_message*>::iterator iter = messages.begin(); iter != messages.end(); iter++)
 	{
-	case sftpReply:
-		LogMessageRaw(Response, event.GetText());
-		ProcessReply(true, event.GetText());
-		break;
-	case sftpStatus:
-	case sftpError:
-	case sftpVerbose:
-		wxFAIL_MSG(_T("given notification codes should have been handled by thread"));
-		break;
-	case sftpDone:
+		if (!m_pInputThread)
 		{
-			ProcessReply(event.GetText() == _T("1"));
-			break;
+			delete *iter;
+			continue;
 		}
-	case sftpRequestPreamble:
-		m_requestPreamble = event.GetText();
-		break;
-	case sftpRequestInstruction:
-		m_requestInstruction = event.GetText();
-		break;
-	case sftpRequest:
-		switch(event.GetRequestType())
+
+		sftp_message* message = *iter;
+
+		switch (message->type)
 		{
-		case sftpReqPassword:
-			if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
+		case sftpReply:
+			LogMessageRaw(Response, message->text);
+			ProcessReply(true, message->text);
+			break;
+		case sftpStatus:
+			LogMessageRaw(Status, message->text);
+			break;
+		case sftpError:
+			LogMessageRaw(::Error, message->text);
+			break;
+		case sftpVerbose:
+			LogMessageRaw(Debug_Info, message->text);
+			break;
+		case sftpDone:
 			{
-				LogMessage(Debug_Warning, _T("sftpReqPassword outside connect operation, ignoring."));
+				ProcessReply(message->text == _T("1"));
 				break;
 			}
-
-			if (m_pCurrentServer->GetLogonType() == INTERACTIVE)
+		case sftpRequestPreamble:
+			m_requestPreamble = message->text;
+			break;
+		case sftpRequestInstruction:
+			m_requestInstruction = message->text;
+			break;
+		case sftpRequest:
+			switch(message->reqType)
 			{
-				wxString challenge;
-				if (m_requestPreamble != _T(""))
-					challenge += m_requestPreamble + _T("\n");
-				if (m_requestInstruction != _T(""))
-					challenge += m_requestInstruction + _T("\n");
-				if (event.GetText() != _T("Password:"))
-					challenge += event.GetText();
-				CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(challenge);
-				pNotification->server = *m_pCurrentServer;
-				pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
-				m_pEngine->AddNotification(pNotification);
-			}
-			else
-			{
-				CSftpConnectOpData *pData = reinterpret_cast<CSftpConnectOpData*>(m_pCurOpData);
-
-				const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction;
-
-				if (pData->pLastChallenge)
+			case sftpReqPassword:
+				if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
 				{
-					// Check for same challenge. Will most likely fail as well, so abort early.
-					if (*pData->pLastChallenge == newChallenge)
-					{
-						LogMessage(::Error, _T("Authentication failed."));
-						DoClose(FZ_REPLY_CRITICALERROR);
-						return;
-					}
-					delete pData->pLastChallenge;
+					LogMessage(Debug_Warning, _T("sftpReqPassword outside connect operation, ignoring."));
+					break;
 				}
-				
-				pData->pLastChallenge = new wxString(newChallenge);
-				
-				const wxString pass = m_pCurrentServer->GetPass();
-				wxString show = _T("Pass: ");
-				show.Append('*', pass.Length());
-				Send(pass, show);
+
+				if (m_pCurrentServer->GetLogonType() == INTERACTIVE)
+				{
+					wxString challenge;
+					if (m_requestPreamble != _T(""))
+						challenge += m_requestPreamble + _T("\n");
+					if (m_requestInstruction != _T(""))
+						challenge += m_requestInstruction + _T("\n");
+					if (message->text != _T("Password:"))
+						challenge += message->text;
+					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(challenge);
+					pNotification->server = *m_pCurrentServer;
+					pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
+					m_pCurOpData->waitForAsyncRequest = true;
+					m_pEngine->AddNotification(pNotification);
+				}
+				else
+				{
+					CSftpConnectOpData *pData = reinterpret_cast<CSftpConnectOpData*>(m_pCurOpData);
+
+					const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction;
+
+					if (pData->pLastChallenge)
+					{
+						// Check for same challenge. Will most likely fail as well, so abort early.
+						if (*pData->pLastChallenge == newChallenge)
+						{
+							LogMessage(::Error, _T("Authentication failed."));
+							DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
+							return;
+						}
+						delete pData->pLastChallenge;
+					}
+
+					pData->pLastChallenge = new wxString(newChallenge);
+
+					const wxString pass = m_pCurrentServer->GetPass();
+					wxString show = _T("Pass: ");
+					show.Append('*', pass.Length());
+					Send(pass, show);
+				}
+				break;
+			default:
+				wxFAIL_MSG(_T("given notification codes should have been handled by thread"));
+				break;
 			}
 			break;
+		case sftpListentry:
+			ListParseEntry(message->text);
+			break;
+		case sftpRead:
+		case sftpWrite:
+			{
+				long number = 0;
+				message->text.ToLong(&number);
+				UpdateTransferStatus(number);
+			}
+			break;
+		case sftpRecv:
+			SetActive(true);
+			break;
+		case sftpSend:
+			SetActive(false);
+			break;
+		case sftpUsedQuotaRecv:
+			OnQuotaRequest(CRateLimiter::inbound);
+			break;
+		case sftpUsedQuotaSend:
+			OnQuotaRequest(CRateLimiter::outbound);
+			break;
 		default:
-			wxFAIL_MSG(_T("given notification codes should have been handled by thread"));
+			wxFAIL_MSG(_T("given notification codes not handled"));
 			break;
 		}
-		break;
-	case sftpListentry:
-		ListParseEntry(event.GetText());
-		break;
-	case sftpRead:
-	case sftpWrite:
-		UpdateTransferStatus(event.GetNumber());
-		break;
-	case sftpUsedQuotaRecv:
-		OnQuotaRequest(CRateLimiter::inbound);
-		break;
-	case sftpUsedQuotaSend:
-		OnQuotaRequest(CRateLimiter::outbound);
-		break;
-	default:
-		wxFAIL_MSG(_T("given notification codes not handled"));
-		break;
+		delete message;
 	}
 }
 
@@ -555,12 +599,19 @@ bool CSftpControlSocket::Send(wxString cmd, const wxString& show /*=_T("")*/)
 	return AddToStream(cmd);
 }
 
-bool CSftpControlSocket::AddToStream(const wxString& cmd)
+bool CSftpControlSocket::AddToStream(const wxString& cmd, bool force_utf8 /*=false*/)
 {
-	const wxCharBuffer str = ConvToServer(cmd);
+	const wxCharBuffer str = ConvToServer(cmd, force_utf8);
+
 	if (!m_pProcess)
 		return false;
 
+	if (!str)
+	{
+		LogMessage(::Error, _("Could not convert command to server encoding"));
+		return false;
+	}
+
 	wxOutputStream* pStream = m_pProcess->GetOutputStream();
 	if (!pStream)
 		return false;
@@ -660,7 +711,8 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 
 					CDirectoryListing listing;
 					CDirectoryCache cache;
-					bool found = cache.Lookup(listing, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, true);
+					bool is_outdated = false;
+					bool found = cache.Lookup(listing, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, true, is_outdated);
 					if (found)
 					{
 						bool differentCase = false;
@@ -673,7 +725,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 									differentCase = true;
 								else
 								{
-									wxLongLong size = listing[i].size;	
+									wxLongLong size = listing[i].size;
 									pData->remoteFileSize = size.GetLo() + ((wxFileOffset)size.GetHi() << 32);
 									found = true;
 									break;
@@ -742,7 +794,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	case reqId_interactiveLogin:
 		{
 			CInteractiveLoginNotification *pInteractiveLoginNotification = reinterpret_cast<CInteractiveLoginNotification *>(pNotification);
-		
+
 			if (!pInteractiveLoginNotification->passwordSet)
 			{
 				DoClose(FZ_REPLY_CANCELED);
@@ -827,34 +879,7 @@ int CSftpControlSocket::List(CServerPath path /*=CServerPath()*/, wxString subDi
 	if (res != FZ_REPLY_OK)
 		return res;
 
-	if (!pData->refresh)
-	{
-		wxASSERT(!pData->pNextOpData);
-
-		// Do a cache lookup now that we know the correct directory
-		CDirectoryCache cache;
-
-		int hasUnsureEntries;
-		bool found = cache.DoesExist(*m_pCurrentServer, m_CurrentPath, _T(""), hasUnsureEntries);
-		if (found)
-		{
-			if (!pData->path.IsEmpty() && pData->subDir != _T(""))
-				cache.AddParent(*m_pCurrentServer, m_CurrentPath, pData->path, pData->subDir);
-
-			m_pEngine->SendDirectoryListingNotification(m_CurrentPath, true, false, false);
-			ResetOperation(FZ_REPLY_OK);
-
-			return FZ_REPLY_OK;
-		}
-	}
-
-	// Try to lock cache
-	if (!TryLockCache(m_CurrentPath))
-		return FZ_REPLY_WOULDBLOCK;
-
-	pData->opState = list_list;
-
-	return ListSend();
+	return ParseSubcommandResult(FZ_REPLY_OK);
 }
 
 int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply)
@@ -964,9 +989,9 @@ int CSftpControlSocket::ListParseEntry(const wxString& entry)
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::ListSend(int prevResult /*=FZ_REPLY_OK*/)
+int CSftpControlSocket::ListSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ListSend(%d)"), prevResult);
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ListSubcommandResult()"));
 
 	if (!m_pCurOpData)
 	{
@@ -978,54 +1003,79 @@ int CSftpControlSocket::ListSend(int prevResult /*=FZ_REPLY_OK*/)
 	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
 	LogMessage(Debug_Debug, _T("  state = %d"), pData->opState);
 
-	if (pData->opState == list_waitcwd)
+	if (pData->opState != list_waitcwd)
 	{
-		if (prevResult != FZ_REPLY_OK)
-		{
-			ResetOperation(prevResult);
-			return FZ_REPLY_ERROR;
-		}
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
 
-		if (!pData->refresh)
-		{
-			wxASSERT(!pData->pNextOpData);
+	if (prevResult != FZ_REPLY_OK)
+	{
+		ResetOperation(prevResult);
+		return FZ_REPLY_ERROR;
+	}
 
-			// Do a cache lookup now that we know the correct directory
-			CDirectoryCache cache;
-			int hasUnsureEntries;
-			bool found = cache.DoesExist(*m_pCurrentServer, m_CurrentPath, _T(""), hasUnsureEntries);
-			if (found)
+	if (!pData->refresh)
+	{
+		wxASSERT(!pData->pNextOpData);
+
+		// Do a cache lookup now that we know the correct directory
+		CDirectoryCache cache;
+
+		int hasUnsureEntries;
+		bool is_outdated = false;
+		bool found = cache.DoesExist(*m_pCurrentServer, m_CurrentPath, _T(""), hasUnsureEntries, is_outdated);
+		if (found)
+		{
+			if (!pData->path.IsEmpty() && pData->subDir != _T(""))
+				cache.AddParent(*m_pCurrentServer, m_CurrentPath, pData->path, pData->subDir);
+
+			// We're done if listins is recent and has no outdated entries
+			if (!is_outdated && !hasUnsureEntries)
 			{
-				if (!pData->path.IsEmpty() && pData->subDir != _T(""))
-					cache.AddParent(*m_pCurrentServer, m_CurrentPath, pData->path, pData->subDir);
+				m_pEngine->SendDirectoryListingNotification(m_CurrentPath, true, false, false);
 
-				// Continue with refresh if listing has unsure entries
-				if (!hasUnsureEntries)
-				{
-					m_pEngine->SendDirectoryListingNotification(m_CurrentPath, true, false, false);
+				ResetOperation(FZ_REPLY_OK);
 
-					ResetOperation(FZ_REPLY_OK);
-
-					return FZ_REPLY_OK;
-				}
+				return FZ_REPLY_OK;
 			}
 		}
-
-		if (!HasLock())
-		{
-			if (!TryLockCache(m_CurrentPath))
-				return FZ_REPLY_WOULDBLOCK;
-		}
-
-		pData->opState = list_list;
 	}
 
+	if (!HasLock())
+	{
+		if (!TryLockCache(m_CurrentPath))
+			return FZ_REPLY_WOULDBLOCK;
+	}
+
+	pData->opState = list_list;
+
+	return SendNextCommand();
+}
+
+int CSftpControlSocket::ListSend()
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ListSend()"));
+
+	if (!m_pCurOpData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
+	LogMessage(Debug_Debug, _T("  state = %d"), pData->opState);
+
 	if (pData->opState == list_list)
 	{
 		pData->pParser = new CDirectoryListingParser(this, *m_pCurrentServer);
 		Send(_T("ls"));
 		return FZ_REPLY_WOULDBLOCK;
 	}
+
+	LogMessage(Debug_Warning, _T("Unknown opStatein CSftpControlSocket::ListSend"));
+	ResetOperation(FZ_REPLY_INTERNALERROR);
 	return FZ_REPLY_ERROR;
 }
 
@@ -1104,7 +1154,7 @@ int CSftpControlSocket::ChangeDir(CServerPath path /*=CServerPath()*/, wxString
 
 	m_pCurOpData = pData;
 
-	return ChangeDirSend();
+	return SendNextCommand();
 }
 
 int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString& reply)
@@ -1134,7 +1184,7 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 		if (!successful)
 		{
 			// Create remote directory if part of a file upload
-			if (pData->pNextOpData && pData->pNextOpData->opId == cmd_transfer && 
+			if (pData->pNextOpData && pData->pNextOpData->opId == cmd_transfer &&
 				!static_cast<CSftpFileTransferOpData *>(pData->pNextOpData)->download && !pData->triedMkd)
 			{
 				pData->triedMkd = true;
@@ -1189,11 +1239,20 @@ int CSftpControlSocket::ChangeDirParseResponse(bool successful, const wxString&
 		return FZ_REPLY_ERROR;
 	}
 
-	return ChangeDirSend();
+	return SendNextCommand();
+}
+
+int CSftpControlSocket::ChangeDirSubcommandResult(int WXUNUSED(prevResult))
+{
+	LogMessage(Debug_Verbose, _("CSftpControlSocket::ChangeDirSubcommandResult()"));
+
+	return SendNextCommand();
 }
 
 int CSftpControlSocket::ChangeDirSend()
 {
+	LogMessage(Debug_Verbose, _("CSftpControlSocket::ChangeDirSend()"));
+
 	if (!m_pCurOpData)
 	{
 		ResetOperation(FZ_REPLY_ERROR);
@@ -1271,13 +1330,19 @@ int CSftpControlSocket::ResetOperation(int nErrorCode)
 		if (pData->criticalFailure)
 			nErrorCode |= FZ_REPLY_CRITICALERROR;
 	}
+	if (m_pCurOpData && m_pCurOpData->opId == cmd_delete && !(nErrorCode & FZ_REPLY_DISCONNECTED))
+	{
+		CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
+		if (pData->m_needSendListing)
+			m_pEngine->SendDirectoryListingNotification(pData->path, false, true, false);
+	}
 
 	return CControlSocket::ResetOperation(nErrorCode);
 }
 
-int CSftpControlSocket::SendNextCommand(int prevResult /*=FZ_REPLY_OK*/)
+int CSftpControlSocket::SendNextCommand()
 {
-	LogMessage(Debug_Verbose, _T("CSftpControlSocket::SendNextCommand(%d)"), prevResult);
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::SendNextCommand()"));
 	if (!m_pCurOpData)
 	{
 		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("SendNextCommand called without active operation"));
@@ -1294,13 +1359,19 @@ int CSftpControlSocket::SendNextCommand(int prevResult /*=FZ_REPLY_OK*/)
 	switch (m_pCurOpData->opId)
 	{
 	case cmd_list:
-		return ListSend(prevResult);
+		return ListSend();
 	case cmd_transfer:
-		return FileTransferSend(prevResult);
+		return FileTransferSend();
 	case cmd_cwd:
 		return ChangeDirSend();
 	case cmd_mkdir:
 		return MkdirSend();
+	case cmd_rename:
+		return RenameSend();
+	case cmd_chmod:
+		return ChmodSend();
+	case cmd_delete:
+		return DeleteSend();
 	default:
 		LogMessage(::Debug_Warning, __TFILE__, __LINE__, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
@@ -1364,48 +1435,12 @@ int CSftpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	if (res != FZ_REPLY_OK)
 		return res;
 
-	pData->opState = filetransfer_waitlist;
-
-	CDirentry entry;
-	bool dirDidExist;
-	bool matchedCase;
-	CDirectoryCache cache;
-	bool found = cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dirDidExist, matchedCase);
-	bool shouldList = false;
-	if (!found)
-	{
-		if (!dirDidExist)
-			shouldList = true;
-	}
-	else
-	{
-		if (entry.unsure)
-			shouldList = true;
-		else
-		{
-			if (matchedCase)
-				pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
-		}
-	}
-	if (shouldList)
-	{
-		res = List();
-		if (res != FZ_REPLY_OK)
-			return res;
-	}
-
-	pData->opState = filetransfer_transfer;
-
-	res = CheckOverwriteFile();
-	if (res != FZ_REPLY_OK)
-		return res;
-
-	return FileTransferSend();
+	return ParseSubcommandResult(FZ_REPLY_OK);
 }
 
-int CSftpControlSocket::FileTransferSend(int prevResult /*=FZ_REPLY_OK*/)
+int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("FileTransferSend()"));
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::FileTransferSubcommandResult()"));
 
 	if (!m_pCurOpData)
 	{
@@ -1486,13 +1521,35 @@ int CSftpControlSocket::FileTransferSend(int prevResult /*=FZ_REPLY_OK*/)
 					return res;
 			}
 		}
-		
+
 		pData->opState = filetransfer_transfer;
 
 		int res = CheckOverwriteFile();
 		if (res != FZ_REPLY_OK)
 			return res;
 	}
+	else
+	{
+		LogMessage(Debug_Warning, _T("  Unknown opState (%d)"), pData->opState);
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	return SendNextCommand();
+}
+
+int CSftpControlSocket::FileTransferSend()
+{
+	LogMessage(Debug_Verbose, _T("FileTransferSend()"));
+
+	if (!m_pCurOpData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
 	wxString cmd;
 	if (pData->resume)
@@ -1509,23 +1566,41 @@ int CSftpControlSocket::FileTransferSend(int prevResult /*=FZ_REPLY_OK*/)
 		InitTransferStatus(pData->remoteFileSize, pData->resume ? pData->localFileSize : 0, false);
 		cmd += _T("get ");
 		cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath)) + _T(" ");
-		cmd += QuoteFilename(pData->localFile);
+
+		wxString localFile = QuoteFilename(pData->localFile);
+		wxString logstr = cmd;
+		logstr += localFile;
+		LogMessageRaw(Command, logstr);
+
+		if (!AddToStream(cmd) || !AddToStream(localFile + _T("\n"), true))
+		{
+			ResetOperation(FZ_REPLY_ERROR);
+			return FZ_REPLY_ERROR;
+		}
 	}
 	else
 	{
 		InitTransferStatus(pData->localFileSize, pData->resume ? pData->remoteFileSize : 0, false);
 		cmd += _T("put ");
-		cmd += QuoteFilename(pData->localFile) + _T(" ");
-		cmd += QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
+
+		wxString logstr = cmd;
+		wxString localFile = QuoteFilename(pData->localFile) + _T(" ");
+		wxString remoteFile = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
+
+		logstr += localFile;
+		logstr += remoteFile;
+		LogMessageRaw(Command, logstr);
+
+		if (!AddToStream(cmd) || !AddToStream(localFile, true) ||
+			!AddToStream(remoteFile + _T("\n")))
+		{
+			ResetOperation(FZ_REPLY_ERROR);
+			return FZ_REPLY_ERROR;
+		}
 	}
 	SetTransferStatusStartTime();
 
 	pData->transferInitiated = true;
-	if (!Send(cmd))
-	{
-		ResetOperation(FZ_REPLY_ERROR);
-		return FZ_REPLY_ERROR;
-	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -1611,7 +1686,7 @@ enum mkdStates
 int CSftpControlSocket::Mkdir(const CServerPath& path)
 {
 	/* Directory creation works like this: First find a parent directory into
-	 * which we can CWD, then create the subdirs one by one. If either part 
+	 * which we can CWD, then create the subdirs one by one. If either part
 	 * fails, try MKD with the full path directly.
 	 */
 
@@ -1723,6 +1798,7 @@ int CSftpControlSocket::MkdirParseResponse(bool successful, const wxString& repl
 			ResetOperation(FZ_REPLY_OK);
 			return FZ_REPLY_OK;
 		}
+		break;
 	default:
 		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("unknown op state: %d"), pData->opState);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
@@ -1774,7 +1850,7 @@ int CSftpControlSocket::MkdirSend()
 
 	if (!res)
 		return FZ_REPLY_ERROR;
-	
+
 	return FZ_REPLY_WOULDBLOCK;
 }
 
@@ -1784,44 +1860,19 @@ wxString CSftpControlSocket::QuoteFilename(wxString filename)
 	return _T("\"") + filename + _T("\"");
 }
 
-class CSftpDeleteOpData : public COpData
-	{
-	public:
-		CSftpDeleteOpData()
-			: COpData(cmd_delete)
-		{
-		}
-
-		virtual ~CSftpDeleteOpData() { }
-
-		CServerPath path;
-		wxString file;
-	};
-
-int CSftpControlSocket::Delete(const CServerPath& path /*=CServerPath()*/, const wxString& file /*=_T("")*/)
+int CSftpControlSocket::Delete(const CServerPath& path, const std::list<wxString>& files)
 {
 	LogMessage(Debug_Verbose, _T("CSftpControlSocket::Delete"));
 	wxASSERT(!m_pCurOpData);
 	CSftpDeleteOpData *pData = new CSftpDeleteOpData();
 	m_pCurOpData = pData;
 	pData->path = path;
-	pData->file = file;
+	pData->files = files;
 
-	wxString filename = path.FormatFilename(file);
-	if (filename == _T(""))
-	{
-		LogMessage(::Error, _T("Filename cannot be constructed for folder %s and filename %s"), path.GetPath().c_str(), file.c_str());
-		return FZ_REPLY_ERROR;
-	}
+	// CFileZillaEnginePrivate should have checked this already
+	wxASSERT(!files.empty());
 
-	CDirectoryCache cache;
-	cache.InvalidateFile(*m_pCurrentServer, path, file);
-
-	if (!Send(_T("rm ") + WildcardEscape(QuoteFilename(filename)),
-			  _T("rm ") + QuoteFilename(filename)))
-		return FZ_REPLY_ERROR;
-
-	return FZ_REPLY_WOULDBLOCK;
+	return SendNextCommand();
 }
 
 int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString& reply)
@@ -1835,16 +1886,71 @@ int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString& rep
 		return FZ_REPLY_ERROR;
 	}
 
-	if (!successful)
-		return ResetOperation(FZ_REPLY_ERROR);
-
 	CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
 
-	CDirectoryCache cache;
-	cache.RemoveFile(*m_pCurrentServer, pData->path, pData->file);
-	m_pEngine->SendDirectoryListingNotification(pData->path, false, true, false);
+	if (!successful)
+		pData->m_deleteFailed = true;
+	else
+	{
+		const wxString& file = pData->files.front();
 
-	return ResetOperation(FZ_REPLY_OK);
+		CDirectoryCache cache;
+		cache.RemoveFile(*m_pCurrentServer, pData->path, file);
+
+		wxDateTime now = wxDateTime::UNow();
+		if (now.IsValid() && pData->m_time.IsValid() && (now - pData->m_time).GetSeconds() >= 1)
+		{
+			m_pEngine->SendDirectoryListingNotification(pData->path, false, true, false);
+			pData->m_time = now;
+			pData->m_needSendListing = false;
+		}
+		else
+			pData->m_needSendListing = true;
+	}
+
+	pData->files.pop_front();
+
+	if (!pData->files.empty())
+		return SendNextCommand();
+
+	return ResetOperation(pData->m_deleteFailed ? FZ_REPLY_ERROR : FZ_REPLY_OK);
+}
+
+int CSftpControlSocket::DeleteSend()
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::DeleteSend"));
+
+	if (!m_pCurOpData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+	CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
+
+	const wxString& file = pData->files.front();
+	if (file == _T(""))
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty filename"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	wxString filename = pData->path.FormatFilename(file);
+	if (filename == _T(""))
+	{
+		LogMessage(::Error, _T("Filename cannot be constructed for folder %s and filename %s"), pData->path.GetPath().c_str(), file.c_str());
+		return FZ_REPLY_ERROR;
+	}
+
+	CDirectoryCache cache;
+	cache.InvalidateFile(*m_pCurrentServer, pData->path, file);
+
+	if (!Send(_T("rm ") + WildcardEscape(QuoteFilename(filename)),
+			  _T("rm ") + QuoteFilename(filename)))
+		return FZ_REPLY_ERROR;
+
+	return FZ_REPLY_WOULDBLOCK;
 }
 
 class CSftpRemoveDirOpData : public COpData
@@ -1872,7 +1978,7 @@ int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, co
 	pData->subDir = subDir;
 
 	CServerPath fullPath = pData->path;
-		
+
 	if (!fullPath.AddSegment(subDir))
 	{
 		LogMessage(::Error, _T("Path cannot be constructed for folder %s and subdir %s"), path.GetPath().c_str(), subDir.c_str());
@@ -1885,6 +1991,7 @@ int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, co
 	CPathCache pathCache;
 	pathCache.InvalidatePath(*m_pCurrentServer, pData->path, pData->subDir);
 
+	m_pEngine->InvalidateCurrentWorkingDirs(fullPath);
 	if (!Send(_T("rmdir ") + WildcardEscape(QuoteFilename(fullPath.GetPath())),
 			  _T("rmdir ") + QuoteFilename(fullPath.GetPath())))
 		return FZ_REPLY_ERROR;
@@ -1963,8 +2070,8 @@ int CSftpControlSocket::Chmod(const CChmodCommand& command)
 	int res = ChangeDir(command.GetPath());
 	if (res != FZ_REPLY_OK)
 		return res;
-	
-	return ChmodSend();
+
+	return SendNextCommand();
 }
 
 int CSftpControlSocket::ChmodParseResponse(bool successful, const wxString& reply)
@@ -1987,8 +2094,10 @@ int CSftpControlSocket::ChmodParseResponse(bool successful, const wxString& repl
 	return FZ_REPLY_OK;
 }
 
-int CSftpControlSocket::ChmodSend(int prevResult /*=FZ_REPLY_OK*/)
+int CSftpControlSocket::ChmodSubcommandResult(int prevResult)
 {
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ChmodSend()"));
+
 	CSftpChmodOpData *pData = static_cast<CSftpChmodOpData*>(m_pCurOpData);
 	if (!pData)
 	{
@@ -2000,6 +2109,21 @@ int CSftpControlSocket::ChmodSend(int prevResult /*=FZ_REPLY_OK*/)
 	if (prevResult != FZ_REPLY_OK)
 		pData->m_useAbsolute = true;
 
+	return SendNextCommand();
+}
+
+int CSftpControlSocket::ChmodSend()
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ChmodSend()"));
+
+	CSftpChmodOpData *pData = static_cast<CSftpChmodOpData*>(m_pCurOpData);
+	if (!pData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("m_pCurOpData empty"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
 	bool res;
 	switch (pData->opState)
 	{
@@ -2007,7 +2131,7 @@ int CSftpControlSocket::ChmodSend(int prevResult /*=FZ_REPLY_OK*/)
 		{
 			CDirectoryCache cache;
 			cache.UpdateFile(*m_pCurrentServer, pData->m_cmd.GetPath(), pData->m_cmd.GetFile(), false, CDirectoryCache::unknown);
-		
+
 			wxString quotedFilename = QuoteFilename(pData->m_cmd.GetPath().FormatFilename(pData->m_cmd.GetFile(), !pData->m_useAbsolute));
 
 			res = Send(_T("chmod ") + pData->m_cmd.GetPermission() + _T(" ") + WildcardEscape(quotedFilename),
@@ -2068,8 +2192,8 @@ int CSftpControlSocket::Rename(const CRenameCommand& command)
 	int res = ChangeDir(command.GetFromPath());
 	if (res != FZ_REPLY_OK)
 		return res;
-	
-	return RenameSend();
+
+	return SendNextCommand();
 }
 
 int CSftpControlSocket::RenameParseResponse(bool successful, const wxString& reply)
@@ -2102,8 +2226,10 @@ int CSftpControlSocket::RenameParseResponse(bool successful, const wxString& rep
 	return FZ_REPLY_OK;
 }
 
-int CSftpControlSocket::RenameSend(int prevResult /*=FZ_REPLY_OK*/)
+int CSftpControlSocket::RenameSubcommandResult(int prevResult)
 {
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::RenameSubcommandResult()"));
+
 	CSftpRenameOpData *pData = static_cast<CSftpRenameOpData*>(m_pCurOpData);
 	if (!pData)
 	{
@@ -2115,24 +2241,52 @@ int CSftpControlSocket::RenameSend(int prevResult /*=FZ_REPLY_OK*/)
 	if (prevResult != FZ_REPLY_OK)
 		pData->m_useAbsolute = true;
 
+	return SendNextCommand();
+}
+
+int CSftpControlSocket::RenameSend()
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::RenameSend()"));
+
+	CSftpRenameOpData *pData = static_cast<CSftpRenameOpData*>(m_pCurOpData);
+	if (!pData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("m_pCurOpData empty"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
 	bool res;
 	switch (pData->opState)
 	{
 	case rename_rename:
 		{
 			CDirectoryCache cache;
-			cache.InvalidateFile(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile());
+			bool wasDir = false;
+			cache.InvalidateFile(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile(), &wasDir);
 			cache.InvalidateFile(*m_pCurrentServer, pData->m_cmd.GetToPath(), pData->m_cmd.GetToFile());
 
-			wxString localQuoted = QuoteFilename(pData->m_cmd.GetFromPath().FormatFilename(pData->m_cmd.GetFromFile(), !pData->m_useAbsolute));
-			wxString remoteQuoted = QuoteFilename(pData->m_cmd.GetToPath().FormatFilename(pData->m_cmd.GetToFile(), !pData->m_useAbsolute && pData->m_cmd.GetFromPath() == pData->m_cmd.GetToPath()));
+			wxString fromQuoted = QuoteFilename(pData->m_cmd.GetFromPath().FormatFilename(pData->m_cmd.GetFromFile(), !pData->m_useAbsolute));
+			wxString toQuoted = QuoteFilename(pData->m_cmd.GetToPath().FormatFilename(pData->m_cmd.GetToFile(), !pData->m_useAbsolute && pData->m_cmd.GetFromPath() == pData->m_cmd.GetToPath()));
 
 			CPathCache pathCache;
 			pathCache.InvalidatePath(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile());
 			pathCache.InvalidatePath(*m_pCurrentServer, pData->m_cmd.GetToPath(), pData->m_cmd.GetToFile());
 
-			res = Send(_T("mv ") + WildcardEscape(localQuoted) + _T(" ") + remoteQuoted,
-					   _T("mv ") + localQuoted + _T(" ") + remoteQuoted);
+			if (wasDir)
+			{
+				// Need to invalidate current working directories
+				CServerPath path = pathCache.Lookup(*m_pCurrentServer, pData->m_cmd.GetFromPath(), pData->m_cmd.GetFromFile());
+				if (path.IsEmpty())
+				{
+					path = pData->m_cmd.GetFromPath();
+					path.AddSegment(pData->m_cmd.GetFromFile());
+				}
+				m_pEngine->InvalidateCurrentWorkingDirs(path);
+			}
+
+			res = Send(_T("mv ") + WildcardEscape(fromQuoted) + _T(" ") + toQuoted,
+					   _T("mv ") + fromQuoted + _T(" ") + toQuoted);
 		}
 		break;
 	default:
@@ -2158,7 +2312,7 @@ wxString CSftpControlSocket::WildcardEscape(const wxString& file)
 	escapedFile.Alloc(file.Len());
 	for (unsigned int i = 0; i < file.Len(); i++)
 	{
-		const char& c = file[i];
+		const wxChar& c = file[i];
 		switch (c)
 		{
 		case '[':
@@ -2194,3 +2348,35 @@ void CSftpControlSocket::OnQuotaRequest(enum CRateLimiter::rate_direction direct
 	else
 		Wait(direction);
 }
+
+
+int CSftpControlSocket::ParseSubcommandResult(int prevResult)
+{
+	LogMessage(Debug_Verbose, _T("CSftpControlSocket::ParseSubcommandResult(%d)"), prevResult);
+	if (!m_pCurOpData)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("ParseSubcommandResult called without active operation"));
+		ResetOperation(FZ_REPLY_ERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	switch (m_pCurOpData->opId)
+	{
+	case cmd_cwd:
+		return ChangeDirSubcommandResult(prevResult);
+	case cmd_list:
+		return ListSubcommandResult(prevResult);
+	case cmd_transfer:
+		return FileTransferSubcommandResult(prevResult);
+	case cmd_rename:
+		return RenameSubcommandResult(prevResult);
+	case cmd_chmod:
+		return ChmodSubcommandResult(prevResult);
+	default:
+		LogMessage(__TFILE__, __LINE__, this, ::Debug_Warning, _T("Unknown opID (%d) in ParseSubcommandResult"), m_pCurOpData->opId);
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		break;
+	}
+
+	return FZ_REPLY_ERROR;
+}
