@@ -87,6 +87,8 @@ bool CTlsSocket::Init()
 		Uninit();
 		return false;
 	}
+
+	gnutls_dh_set_prime_bits(m_session, 512);
 	
 	// Set which type of certificates we accept
 	const int cert_type_priority[] = { GNUTLS_CRT_X509, 0 };
@@ -247,7 +249,7 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	{
 		if (m_pSocketBackend->LastError() == wxSOCKET_WOULDBLOCK)
 		{
-			if (m_canCheckCloseSocket)
+			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound))
 			{
 				wxSocketEvent evt(m_pSocketBackend->GetId());
 				evt.m_event = wxSOCKET_LOST;
