@@ -31,14 +31,18 @@
 #include <wx/xrc/xh_stlin.h>
 #include <wx/xrc/xh_sttxt.h>
 #include <wx/xrc/xh_text.h>
-#include <wx/xrc/xh_toolb.h>
 #include <wx/xrc/xh_tree.h>
 #include <wx/xrc/xh_hyperlink.h>
+#include "xh_toolb_ex.h"
 
 #if defined(__WXMAC__) || defined(__UNIX__)
 #include <wx/stdpaths.h>
 #endif
 
+#ifdef __WXGTK__
+#include "locale_initializer.h"
+#endif
+
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
 	#include "prefix.h"
@@ -57,7 +61,11 @@
 #define new DEBUG_NEW
 #endif
 
+#ifndef __WXGTK__
 IMPLEMENT_APP(CFileZillaApp);
+#else
+IMPLEMENT_APP_NO_MAIN(CFileZillaApp);
+#endif //__WXGTK__
 
 CFileZillaApp::CFileZillaApp()
 {
@@ -147,15 +155,62 @@ bool CFileZillaApp::OnInit()
 
 	LoadLocales();
 	InitDefaultsDir();
-    InitSettingsDir();
+	InitSettingsDir();
 
 	COptions::Init();
+
 	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
+	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
 	if (language != _T(""))
 	{
-		const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
+#ifdef __WXGTK__
+		if (CInitializer::error)
+		{
+			wxString error;
+
+			wxLocale *loc = wxGetLocale();
+			if (!loc)
+			{
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language."),
+						language.c_str());
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language."),
+						pInfo->Description.c_str(), language.c_str());
+			}
+			else
+			{
+				const wxLanguageInfo* currentInfo = loc->GetLanguageInfo(loc->GetLanguage());
+				wxString currentName;
+				if (currentInfo)
+					currentName = currentInfo->CanonicalName;
+
+				if (currentInfo)
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
+						language.c_str(), loc->GetLocale(),
+						currentName.c_str());
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
+						pInfo->Description.c_str(), language.c_str(), loc->GetLocale(),
+						currentName.c_str());
+			}
+
+			error += _T("\n");
+			error += _("Please make sure the requested locale is installed on your system.");
+			wxMessageBox(error, _("Failed to change language"), wxICON_EXCLAMATION);
+
+			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
+		}
+#else
 		if (!pInfo || !SetLocale(pInfo->Language))
-			wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+		{
+			if (pInfo && pInfo->Description)
+				wxMessageBox(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description.c_str(), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+			else
+				wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+		}
+#endif
 	}
 
 #ifndef _DEBUG
@@ -260,8 +315,18 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 * Basically we just check a couple of paths for presence of the resources,
 	 * and hope we find them. If not, the user can still specify on the cmdline
 	 * and using environment variables where the resources are.
+	 *
+	 * At least on OS X it's simple: All inside application bundle.
 	 */
 
+#ifdef __WXMAC__
+	wxString path = wxStandardPaths::Get().GetDataDir();
+	if (FileExists(path + fileToFind))
+		return path;
+
+	return _T("");
+#else
+
 	wxPathList pathList;
 	// FIXME: --datadir cmdline
 
@@ -270,7 +335,6 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 	// Next try the current path and the current executable path.
 	// Without this, running development versions would be difficult.
-
 	pathList.Add(wxGetCwd());
 
 #ifdef ENABLE_BINRELOC
@@ -299,13 +363,12 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 		wxFileName fn(path);
 		pathList.Add(fn.GetPath());
 	}
-#endif
+#endif //ENABLE_BINRELOC and __WXMSW__ blocks
 
 	// Now scan through the path
 	pathList.AddEnvList(_T("PATH"));
 
-#ifdef __WXMSW__
-#else
+#ifndef __WXMSW__
 	// Try some common paths
 	pathList.Add(_T("/usr/share/filezilla"));
 	pathList.Add(_T("/usr/local/share/filezilla"));
@@ -322,10 +385,6 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 			return cur + _T("/share/filezilla");
 		if (FileExists(cur + _T("/filezilla") + fileToFind))
 			return cur + _T("/filezilla");
-#if defined(__WXMAC__) || defined(__WXCOCOA__)
-		if (FileExists(cur + _T("/FileZilla.app/Contents/SharedSupport") + fileToFind))
-			return cur + _T("/FileZilla.app/Contents/SharedSupport");
-#endif
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); node++)
@@ -345,6 +404,7 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	}
 
 	return _T("");
+#endif //__WXMAC__
 }
 
 bool CFileZillaApp::LoadResourceFiles()
@@ -371,31 +431,31 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
 #endif
 
-    pResource->AddHandler(new wxMenuXmlHandler);
-    pResource->AddHandler(new wxMenuBarXmlHandler);
+	pResource->AddHandler(new wxMenuXmlHandler);
+	pResource->AddHandler(new wxMenuBarXmlHandler);
 	pResource->AddHandler(new wxDialogXmlHandler);
 	pResource->AddHandler(new wxPanelXmlHandler);
 	pResource->AddHandler(new wxSizerXmlHandler);
 	pResource->AddHandler(new wxButtonXmlHandler);
 	pResource->AddHandler(new wxBitmapButtonXmlHandler);
-    pResource->AddHandler(new wxStaticTextXmlHandler);
-    pResource->AddHandler(new wxStaticBoxXmlHandler);
-    pResource->AddHandler(new wxStaticBitmapXmlHandler);
-    pResource->AddHandler(new wxTreeCtrlXmlHandler);
-    pResource->AddHandler(new wxListCtrlXmlHandler);
-    pResource->AddHandler(new wxCheckListBoxXmlHandler);
-    pResource->AddHandler(new wxChoiceXmlHandler);
-    pResource->AddHandler(new wxGaugeXmlHandler);
-    pResource->AddHandler(new wxCheckBoxXmlHandler);
-    pResource->AddHandler(new wxSpinCtrlXmlHandler);
-    pResource->AddHandler(new wxRadioButtonXmlHandler);
-    pResource->AddHandler(new wxNotebookXmlHandler);
-    pResource->AddHandler(new wxTextCtrlXmlHandler);
-    pResource->AddHandler(new wxListBoxXmlHandler);
-    pResource->AddHandler(new wxToolBarXmlHandler);
+	pResource->AddHandler(new wxStaticTextXmlHandler);
+	pResource->AddHandler(new wxStaticBoxXmlHandler);
+	pResource->AddHandler(new wxStaticBitmapXmlHandler);
+	pResource->AddHandler(new wxTreeCtrlXmlHandler);
+	pResource->AddHandler(new wxListCtrlXmlHandler);
+	pResource->AddHandler(new wxCheckListBoxXmlHandler);
+	pResource->AddHandler(new wxChoiceXmlHandler);
+	pResource->AddHandler(new wxGaugeXmlHandler);
+	pResource->AddHandler(new wxCheckBoxXmlHandler);
+	pResource->AddHandler(new wxSpinCtrlXmlHandler);
+	pResource->AddHandler(new wxRadioButtonXmlHandler);
+	pResource->AddHandler(new wxNotebookXmlHandler);
+	pResource->AddHandler(new wxTextCtrlXmlHandler);
+	pResource->AddHandler(new wxListBoxXmlHandler);
+	pResource->AddHandler(new wxToolBarXmlHandlerEx);
 	pResource->AddHandler(new wxStaticLineXmlHandler);
-    pResource->AddHandler(new wxScrolledWindowXmlHandler);
-    pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
+	pResource->AddHandler(new wxScrolledWindowXmlHandler);
+	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
 
 	wxString resourceDir = m_resourceDir;
 #if wxUSE_FILESYSTEM
@@ -410,7 +470,16 @@ bool CFileZillaApp::LoadResourceFiles()
 
 bool CFileZillaApp::InitDefaultsDir()
 {
-	m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
+#ifdef __WXGTK__
+	wxFileName fn = wxFileName(wxGetHomeDir(), _T("fzdefaults.xml"));
+	fn.AppendDir(_T(".filezilla"));
+	if (fn.FileExists())
+		m_defaultsDir = fn.GetPath();
+	else if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml")))
+		m_defaultsDir = _T("/etc/filezilla");
+	else
+#endif
+		m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
 
 	return m_defaultsDir != _T("");
 }
@@ -509,6 +578,7 @@ bool CFileZillaApp::InitSettingsDir()
 
 bool CFileZillaApp::LoadLocales()
 {
+#ifndef __WXMAC__
 	m_localesDir = GetDataDir(_T("/../locale/*/filezilla.mo"));
 	if (m_localesDir != _T(""))
 	{
@@ -528,6 +598,9 @@ bool CFileZillaApp::LoadLocales()
 			m_localesDir += _T("locales");
 		}
 	}
+#else
+	m_localesDir = wxStandardPaths::Get().GetDataDir() + _T("/locales");
+#endif
 
 	if (m_localesDir != _T(""))
 	{
@@ -582,6 +655,13 @@ int CFileZillaApp::GetCurrentLanguage() const
 	return m_pLocale->GetLanguage();
 }
 
+wxString CFileZillaApp::GetCurrentLanguageCode() const
+{
+	if (!m_pLocale)
+		return _T("");
+
+	return m_pLocale->GetCanonicalName();
+}
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 
@@ -632,6 +712,20 @@ void CFileZillaApp::CheckExistsFzsftp()
 {
 	// Get the correct path to the fzsftp executable
 
+#ifdef __WXMAC__
+	wxString executable = wxStandardPaths::Get().GetExecutablePath();
+	int pos = executable.Find('/', true);
+	if (pos != -1)
+		executable = executable.Left(pos);
+	executable += _T("/fzsftp");
+	if (!wxFileName::FileExists(executable))
+	{
+		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable.c_str()),
+			_("File not found"), wxICON_ERROR);
+	}
+#else
+
+
 	wxString program = _T("fzsftp");
 #ifdef __WXMSW__
 	program += _T(".exe");
@@ -684,16 +778,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 	if (!found)
 	{
-#ifdef __WXMAC__
-		// Get application path within the bundle
-		wxFileName fn(wxStandardPaths::Get().GetDataDir() + _T("/../MacOS/"), program);
-		fn.Normalize();
-		if (fn.FileExists())
-		{
-			executable = fn.GetFullPath();
-			found = true;
-		}
-#elif defined(__UNIX__)
+#ifdef __UNIX__
 		const wxString prefix = ((const wxStandardPaths&)wxStandardPaths::Get()).GetInstallPrefix();
 		if (prefix != _T("/usr/local"))
 		{
@@ -732,5 +817,6 @@ void CFileZillaApp::CheckExistsFzsftp()
 			_("File not found"), wxICON_ERROR);
 	}
 	else
+#endif
 		COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
 }
