@@ -4,7 +4,6 @@
 enum interfaceOptions
 {
 	OPTION_NUMTRANSFERS = OPTIONS_ENGINE_NUM,
-	OPTION_TRANSFERRETRYCOUNT,
 	OPTION_ASCIIBINARY,
 	OPTION_ASCIIFILES,
 	OPTION_ASCIINOEXT,
@@ -35,9 +34,25 @@ enum interfaceOptions
 	OPTION_QUEUE_COLUMN_WIDTHS,
 	OPTION_LOCALFILELIST_COLUMN_WIDTHS,
 	OPTION_REMOTEFILELIST_COLUMN_WIDTHS,
-	OPTION_WINDOW_POSITION,
+	OPTION_MAINWINDOW_POSITION,
+	OPTION_MAINWINDOW_SPLITTER_POSITION,
 	OPTION_LOCALFILELIST_SORTORDER,
 	OPTION_REMOTEFILELIST_SORTORDER,
+	OPTION_TIME_FORMAT,
+	OPTION_DATE_FORMAT,
+	OPTION_SHOW_MESSAGELOG,
+	OPTION_SHOW_QUEUE,
+	OPTION_SIZE_FORMAT,
+	OPTION_SIZE_USETHOUSANDSEP,
+	OPTION_EDIT_DEFAULTEDITOR,
+	OPTION_EDIT_ALWAYSDEFAULT,
+	OPTION_EDIT_INHERITASSOCIATIONS,
+	OPTION_EDIT_CUSTOMASSOCIATIONS,
+	OPTION_COMPARISONMODE,
+	OPTION_COMPARISON_THRESHOLD,
+	OPTION_SITEMANAGER_POSITION,
+	OPTION_THEME_ICONSIZE,
+	OPTION_MESSAGELOG_TIMESTAMP,
 
 	// Has to be last element
 	OPTIONS_NUM
@@ -73,6 +88,7 @@ public:
 
 	void Import(TiXmlElement* pElement);
 
+	// Columns in wxListCtrl
 	void SaveColumnWidths(const wxListCtrl* const pListCtrl, unsigned int optionId);
 	bool ReadColumnWidths(unsigned int optionId, unsigned int count, unsigned long* widths);
 	
