@@ -3,6 +3,7 @@
 
 #include <set>
 #include "dndobjects.h"
+#include <wx/progdlg.h>
 
 struct t_newEntry
 {
@@ -12,6 +13,21 @@ struct t_newEntry
 	wxLongLong size;
 };
 
+enum ActionAfterState
+{
+	ActionAfterState_Disabled,
+	ActionAfterState_Close,
+	ActionAfterState_Disconnect,
+	ActionAfterState_RunCommand,
+	ActionAfterState_ShowMessage,
+	ActionAfterState_PlaySound,
+// On Windows, wx can reboot or shutdown the system as well.
+#ifdef __WXMSW__
+	ActionAfterState_Reboot,
+	ActionAfterState_Shutdown
+#endif
+};
+
 class CStatusLineCtrl;
 struct t_EngineData
 {
@@ -52,7 +68,7 @@ public:
 	virtual ~CQueueView();
 
 	bool QueueFile(const bool queueOnly, const bool download, const wxString& localFile, const wxString& remoteFile,
-				const CServerPath& remotePath, const CServer& server, const wxLongLong size);
+		const CServerPath& remotePath, const CServer& server, const wxLongLong size, enum CEditHandler::fileType edit = CEditHandler::none);
 	bool QueueFiles(const bool queueOnly, const wxString& localPath, const CRemoteDataObject& dataObject);
 	bool QueueFiles(const std::list<t_newEntry> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const int defaultFileExistsAction);
 	bool QueueFolder(bool queueOnly, bool download, const wxString& localPath, const CServerPath& remotePath, const CServer& server);
@@ -97,7 +113,7 @@ protected:
 	bool ProcessFolderItems(int type = -1);
 	void ProcessUploadFolderItems();
 
-	void ProcessReply(t_EngineData& engineData, COperationNotification* pNotification);
+	void ProcessReply(t_EngineData* pEngineData, COperationNotification* pNotification);
 	void SendNextCommand(t_EngineData& engineData);
 
 	enum ResetReason
@@ -108,6 +124,8 @@ protected:
 		retry,
 		remove
 	};
+	
+	enum ActionAfterState GetActionAfterState() const;
 
 	void ResetEngine(t_EngineData& data, const enum ResetReason reason);
 	void DeleteEngines();
@@ -126,9 +144,22 @@ protected:
 	void DisplayQueueSize();
 	void SaveQueue();
 	bool ShouldUseBinaryMode(wxString filename);
+	
+	bool IsActionAfter(enum ActionAfterState);
+	void ActionAfter(bool warned = false);
+#ifdef __WXMSW__
+	void ActionAfterWarnUser(bool shutdown);
+#endif
 
 	void ProcessNotification(t_EngineData* pEngineData, CNotification* pNotification);
 
+	// Called from Process Reply.
+	// After a disconnect, check if there's another idle engine that
+	// is already connected.
+	bool SwitchEngine(t_EngineData** ppEngineData);
+
+	bool IsOtherEngineConnected(t_EngineData* pEngineData);
+
 	t_EngineData* GetIdleEngine(const CServer* pServer = 0);
 
 	std::vector<t_EngineData*> m_engineData;
@@ -159,6 +190,15 @@ protected:
 	int m_activeMode; // 0 inactive, 1 only immediate transfers, 2 all
 	bool m_quit;
 
+	enum ActionAfterState m_actionAfterState;
+	wxString m_actionAfterRunCommand;
+#ifdef __WXMSW__
+	wxTimer* m_actionAfterTimer;
+	wxProgressDialog* m_actionAfterWarnDialog;
+	int m_actionAfterTimerCount;
+	int m_actionAfterTimerId;
+#endif
+
 	wxLongLong m_totalQueueSize;
 	int m_filesWithUnknownSize;
 
@@ -169,15 +209,12 @@ protected:
 
 	std::list<CFileZillaEngine*> m_waitingForPassword;
 
-	DECLARE_EVENT_TABLE();
+	virtual void OnPostScroll();
 
+	DECLARE_EVENT_TABLE();
 	void OnEngineEvent(wxEvent &event);
 	void OnFolderThreadComplete(wxCommandEvent& event);
 	void OnFolderThreadFiles(wxCommandEvent& event);
-	void OnScrollEvent(wxScrollWinEvent& event);
-	void OnUpdateStatusLines(wxCommandEvent& event);
-	void OnMouseWheel(wxMouseEvent& event);
-	void OnFocusItemChanged(wxListEvent& event);
 
 	// Context menu handlers
 	void OnContextMenu(wxContextMenuEvent& event);
@@ -192,6 +229,9 @@ protected:
 	void OnSetPriority(wxCommandEvent& event);
 
 	void OnExclusiveEngineRequestGranted(wxCommandEvent& event);
+
+	void OnActionAfter(wxCommandEvent& event);
+	void OnActionAfterTimerTick();
 };
 
 #endif
