@@ -221,6 +221,7 @@ EVT_MENU(XRCID("ID_RENAME"), CRemoteTreeView::OnMenuRename)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnEndLabelEdit)
 EVT_MENU(XRCID("ID_MKDIR"), CRemoteTreeView::OnMkdir)
+EVT_CHAR(CRemoteTreeView::OnChar)
 END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
@@ -320,9 +321,14 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 	wxASSERT(path.GetPath() != _T(""));
 	pieces.push_front(path.GetPath());
 
-	wxTreeItemId parent = GetRootItem();
+	const wxTreeItemId root = GetRootItem();
+	wxTreeItemId parent = root;
+
 	for (std::list<wxString>::const_iterator iter = pieces.begin(); iter != pieces.end(); iter++)
 	{
+		if (parent != root)
+			ListExpand(parent);
+
 		if (iter != pieces.begin())
 			path.AddSegment(*iter);
 
@@ -438,7 +444,7 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 		if (!listing[i].dir)
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, true, -1, false))
+		if (filter.FilenameFiltered(listing[i].name, true, -1, false, 0))
 			continue;
 
 		return true;
@@ -457,7 +463,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 		if (!listing[i].dir)
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, true, -1, false))
+		if (filter.FilenameFiltered(listing[i].name, true, -1, false, 0))
 			continue;
 
 		const wxString& name = listing[i].name;
@@ -514,7 +520,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		if (!listing[i].dir)
 			continue;
 
-		if (!filter.FilenameFiltered(listing[i].name, true, -1, false))
+		if (!filter.FilenameFiltered(listing[i].name, true, -1, false, 0))
 			dirs.push_back(listing[i].name);
 	}
 
@@ -549,25 +555,18 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		}
 		else if (cmp > 0)
 		{
+			// Child no longer exists
 			wxTreeItemId sel = GetSelection();
-			if (sel)
-			{
-				do
-				{
-					sel = GetItemParent(sel);
-					if (sel == parent)
-						break;
-				} while (sel);
-			}
+			while (sel && sel != child)
+				sel = GetItemParent(sel);
+			wxTreeItemId prev = GetPrevSibling(child);
 			if (!sel)
-			{
-				wxTreeItemId prev = GetPrevSibling(child);
 				Delete(child);
-				child = prev;
-			}
+			child = prev;
 		}
 		else if (cmp < 0)
 		{
+			// New directory
 			CServerPath path = listing.path;
 			path.AddSegment(*iter);
 
@@ -593,22 +592,14 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 	}
 	while (child)
 	{
+		// Child no longer exists
 		wxTreeItemId sel = GetSelection();
-		if (sel)
-		{
-			do
-			{
-				sel = GetItemParent(sel);
-				if (sel == parent)
-					break;
-			} while (sel);
-		}
-		if (!sel)
-		{
-			wxTreeItemId prev = GetPrevSibling(child);
+		while (sel && sel != child)
+			sel = GetItemParent(sel);
+		wxTreeItemId prev = GetPrevSibling(child);
+		if (!sel)			
 			Delete(child);
-			child = prev;
-		}
+		child = prev;
 	}
 	while (iter != dirs.rend())
 	{
@@ -658,30 +649,12 @@ void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 	if (!item)
 		return;
 
-	const CItemData* data = (CItemData*)GetItemData(item);
-	wxASSERT(data);
-	if (!data)
-		return;
-
-	CDirectoryListing listing;
-	if (m_pState->m_pEngine->CacheLookup(data->m_path, listing) == FZ_REPLY_OK)
-		RefreshItem(item, listing);
-	else
+	if (!ListExpand(item))
 	{
-		SetItemImages(item, true);
-
-		wxTreeItemId child = GetLastChild(item);
-		if (!child)
-		{
-			event.Veto();
-			return;
-		}
-		if (GetItemText(child) == _T(""))
-		{
-			DeleteChildren(item);
-			event.Veto();
-		}
+		event.Veto();
+		return;
 	}
+
 	Refresh(false);
 }
 
@@ -879,8 +852,13 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 	}
 	else if (!path.HasParent())
-	{
 		pMenu->Enable(XRCID("ID_RENAME"), false);
+
+	const wxString& localDir = m_pState->GetLocalDir();
+	if (!m_pState->LocalDirIsWriteable(localDir))
+	{
+		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
+		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
 
 	PopupMenu(pMenu);
@@ -987,11 +965,26 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, hasParent ? path.GetParent() : path, !cached, currentPath);
 	}
 	else
+	{
 		pChmodDlg->Destroy();
+		const wxTreeItemId selected = GetSelection();
+		if (selected)
+		{
+			CServerPath currentPath = GetPathFromItem(selected);
+			m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
+		}
+	}
 }
 
 void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 {
+	const wxString& localDir = m_pState->GetLocalDir();
+	if (!m_pState->LocalDirIsWriteable(localDir))
+	{
+		wxBell();
+		return;
+	}
+
 	if (!m_pState->IsRemoteIdle())
 		return;
 
@@ -1004,8 +997,6 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const bool hasParent = path.HasParent();
 
-	const wxString localDir = m_pState->GetLocalDir();
-
 	const wxString& name = GetItemText(m_contextMenuItem);
 
 	wxFileName fn = wxFileName(localDir, _T(""));
@@ -1037,6 +1028,9 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 	if (path.IsEmpty())
 		return;
 
+	if (wxMessageBox(_("Really delete all selected files and/or directories?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+		return;
+
 	const bool hasParent = path.HasParent();
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
@@ -1058,7 +1052,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (!currentPath.IsEmpty() && path.IsParentOf(currentPath, false))
+	if (!currentPath.IsEmpty() && (path == currentPath || path.IsParentOf(currentPath, false)))
 		currentPath = startDir;
 
 	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, !hasParent, currentPath);
@@ -1118,7 +1112,14 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
-	const CServerPath& path = GetPathFromItem(event.GetItem());
+	CItemData* const pData = (CItemData*)GetItemData(event.GetItem());
+	if (!pData)
+	{
+		event.Veto();
+		return;
+	}
+
+	const CServerPath path = pData->m_path;
 	if (path.IsEmpty())
 	{
 		event.Veto();
@@ -1141,6 +1142,11 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		return;
 	}
 
+	// Update the item data
+	wxASSERT(path.GetLastSegment() == oldName);
+	pData->m_path = parent;
+	pData->m_path.AddSegment(newName);
+
 	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(parent, oldName, parent, newName));
 	m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(parent));
 
@@ -1203,7 +1209,7 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 		wxString pathName = newPath.GetPath();
 		int pos = pathName.Find(tmpName);
 		wxASSERT(pos != -1);
-		wxString newName = _("New folder");
+		wxString newName = _("New directory");
 		pathName.Replace(tmpName, newName);
 		dlg.SetValue(pathName);
 		dlg.SelectText(pos, pos + newName.Length());
@@ -1234,3 +1240,38 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	if (!currentPath.IsEmpty() && currentPath != listed)
 		m_pState->m_pCommandQueue->ProcessCommand(new CListCommand(currentPath));
 }
+
+bool CRemoteTreeView::ListExpand(wxTreeItemId item)
+{
+	const CItemData* data = (CItemData*)GetItemData(item);
+	wxASSERT(data);
+	if (!data)
+		return false;
+
+	CDirectoryListing listing;
+	if (m_pState->m_pEngine->CacheLookup(data->m_path, listing) == FZ_REPLY_OK)
+		RefreshItem(item, listing);
+	else
+	{
+		SetItemImages(item, true);
+
+		wxTreeItemId child = GetLastChild(item);
+		if (!child || GetItemText(child) == _T(""))
+			return false;
+	}
+
+	return true;
+}
+
+void CRemoteTreeView::OnChar(wxKeyEvent& event)
+{
+	m_contextMenuItem = GetSelection();
+
+	wxCommandEvent cmdEvt;
+	if (event.GetKeyCode() == WXK_F2)
+		OnMenuRename(cmdEvt);
+	else if (event.GetKeyCode() == WXK_DELETE)
+		OnMenuDelete(cmdEvt);
+	else
+		event.Skip();
+}
