@@ -9,12 +9,14 @@ public:
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification *pNotification);
-	void AddToLog(enum MessageType messagetype, wxString message);
+	void AddToLog(enum MessageType messagetype, const wxString& message, const wxDateTime& time);
 
 	void InitDefAttr();
 
 	virtual void SetFocus();
 
+	virtual bool Show(bool show = true);
+
 protected:
 
 	int m_nLineCount;
@@ -38,6 +40,22 @@ protected:
 	} m_attributeCache[MessageTypeCount];
 
 	bool m_rtl;
+	
+	bool m_shown;
+
+	// Don't update actual log window if not shown,
+	// do it later when showing the window.
+	struct t_line
+	{
+		MessageType messagetype;
+		wxString message;
+		wxDateTime time;
+	};
+	std::list<t_line> m_hiddenLines;
+
+	bool m_showTimestamps;
+	wxDateTime m_lastTime;
+	wxString m_lastTimeString;
 };
 
 #endif
