@@ -18,6 +18,8 @@ bool COptionsPageInterface::LoadPage()
 	SetChoice(XRCID("ID_FILEPANELAYOUT"), m_pOptions->GetOptionVal(OPTION_FILEPANE_LAYOUT), failure);
 	SetChoice(XRCID("ID_SORTMODE"), m_pOptions->GetOptionVal(OPTION_FILELIST_DIRSORT), failure);
 
+	SetTextFromOption(XRCID("ID_COMPARISON_THRESHOLD"), OPTION_COMPARISON_THRESHOLD, failure); 
+
 	return !failure;
 }
 
@@ -27,11 +29,18 @@ bool COptionsPageInterface::SavePage()
 	m_pOptions->SetOption(OPTION_FILEPANE_LAYOUT, GetChoice(XRCID("ID_FILEPANELAYOUT")));
 	m_pOptions->SetOption(OPTION_FILELIST_DIRSORT, GetChoice(XRCID("ID_SORTMODE")));
 
+	SetOptionFromText(XRCID("ID_COMPARISON_THRESHOLD"), OPTION_COMPARISON_THRESHOLD); 
+
 	return true;
 }
 
 bool COptionsPageInterface::Validate()
 {
+	wxString text = GetText(XRCID("ID_COMPARISON_THRESHOLD"));
+	long minutes = 1;
+	if (!text.ToLong(&minutes) || minutes < 0 || minutes > 1440)
+		return DisplayError(_T("ID_COMPARISON_THRESHOLD"), _("Comparison threshold needs to be between 0 and 1440 minutes."));
+
 	return true;
 }
 
