@@ -172,6 +172,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		if (m_operationMode == recursive_download)
 		{
 			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			m_pState->RefreshLocalFile(fn.GetFullPath());
 		}
 		else if (m_operationMode == recursive_addtoqueue)
 			m_pQueue->QueueFile(true, true, fn.GetFullPath(), _T(""), CServerPath(), *pServer, -1);
@@ -182,6 +183,8 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 	// Is operation restricted to a single child?
 	bool restrict = !dir.restrict.IsEmpty();
 
+	std::list<wxString> filesToDelete;
+	
 	for (int i = pDirectoryListing->GetCount() - 1; i >= 0; i--)
 	{
 		const CDirentry& entry = (*pDirectoryListing)[i];
@@ -191,7 +194,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			if (entry.name != dir.restrict)
 				continue;
 		}
-		else if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false))
+		else if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
 			continue;
 
 		if (entry.dir && !entry.link)
@@ -220,7 +223,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 				}
 				break;
 			case recursive_delete:
-				m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(pDirectoryListing->path, entry.name));
+				filesToDelete.push_back(entry.name);
 				break;
 			default:
 				break;
@@ -242,6 +245,9 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		}
 	}
 
+	if (m_operationMode == recursive_delete && !filesToDelete.empty())
+		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(pDirectoryListing->path, filesToDelete));
+
 	NextOperation();
 }
 
