@@ -278,6 +278,11 @@ void CViewHeader::AddRecentDirectory(const wxString &directory)
 	{
 		wxString dirToRemove = m_recentDirectories.front();
 		m_recentDirectories.pop_front();
+		if (dirToRemove == directory)
+		{
+			m_recentDirectories.push_back(directory);
+			return;
+		}
 
 		int item = m_pComboBox->FindString(dirToRemove, true);
 		if (item != wxNOT_FOUND)
@@ -376,11 +381,20 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent& event)
 	const wxString path = fn.GetPath();
 
 	wxDir dir;
-	if (name == _T("") || path == _T("") || !dir.Open(path) || !dir.IsOpened())
+	if (name == _T("") || path == _T(""))
 	{
 		m_oldValue = str;
 		return;
 	}
+	else
+	{
+		wxLogNull log;
+		if (!dir.Open(path) || !dir.IsOpened())
+		{
+			m_oldValue = str;
+			return;
+		}
+	}
 	wxString found;
 
 	{
@@ -492,11 +506,10 @@ void CLocalViewHeader::OnStateChange(unsigned int event, const wxString& data)
 	m_autoCompletionText = _T("");
 #endif
 
-	const wxString& dir = m_pState->GetLocalDir();
-	m_pComboBox->SetValue(dir);
-	m_pComboBox->SetSelection(m_pComboBox->GetValue().Length(), m_pComboBox->GetValue().Length());
-
+	wxString dir = m_pState->GetLocalDir();
 	AddRecentDirectory(dir);
+	m_pComboBox->SetStringSelection(dir);
+	m_pComboBox->SetSelection(m_pComboBox->GetValue().Length(), m_pComboBox->GetValue().Length());
 }
 
 BEGIN_EVENT_TABLE(CRemoteViewHeader, CViewHeader)
@@ -519,16 +532,22 @@ void CRemoteViewHeader::OnStateChange(unsigned int event, const wxString& data)
 	if (m_path.IsEmpty())
 	{
 		m_pComboBox->SetValue(_T(""));
-		m_pComboBox->Clear();
 		m_pComboBox->Disable();
 	}
 	else
 	{
+		const CServer* const pServer = m_pState->GetServer();
+		if (pServer && *pServer != m_lastServer)
+		{
+			m_pComboBox->Clear();
+			m_recentDirectories.clear();
+			m_lastServer = *pServer;
+		}
 		m_pComboBox->Enable();
-		m_pComboBox->SetValue(m_path.GetPath());
-		m_pComboBox->SetSelection(m_pComboBox->GetValue().Length(), m_pComboBox->GetValue().Length());
-
 		AddRecentDirectory(m_path.GetPath());
+		m_pComboBox->SetStringSelection(m_path.GetPath());
+		wxString path = m_path.GetPath();
+		m_pComboBox->SetSelection(m_pComboBox->GetValue().Length(), m_pComboBox->GetValue().Length());
 	}
 }
 
