@@ -1188,9 +1188,10 @@ int sftp_general_get(struct sftp_command *cmd, int restart, int multiple)
     }
 
     i = 1;
+    /* FZ unused
     while (i < cmd->nwords && cmd->words[i][0] == '-') {
 	if (!strcmp(cmd->words[i], "--")) {
-	    /* finish processing options */
+	    /* finish processing options *//* FZ unused
 	    i++;
 	    break;
 	} else if (!strcmp(cmd->words[i], "-r")) {
@@ -1201,6 +1202,7 @@ int sftp_general_get(struct sftp_command *cmd, int restart, int multiple)
 	}
 	i++;
     }
+    */
 
     if (i >= cmd->nwords) {
 	fzprintf(sftpError, "%s: expects a filename", cmd->words[0]);
@@ -1865,7 +1867,7 @@ static int sftp_cmd_open(struct sftp_command *cmd)
     return 1;
 }
 
-/* BEGINFZ UNUSED
+/* BEGIN FZ UNUSED
 static int sftp_cmd_lcd(struct sftp_command *cmd)
 {
     char *currdir, *errmsg;
@@ -3037,16 +3039,18 @@ int psftp_main(int argc, char *argv[])
      * otherwise been specified, pop it in `userhost' so that
      * `psftp -load sessname' is sufficient to start a session.
      */
+    /* BEGIN FZ UNUSED
     if (!userhost && cfg.host[0] != '\0') {
 	userhost = dupstr(cfg.host);
     }
+    END FZ UNUSED */
 
     /*
      * If a user@host string has already been provided, connect to
      * it now.
      */
     if (userhost) {
-	fzprintf(sftpVerbose, "pstp: Using userhost passed on commandline: %s", userhost);
+	fzprintf(sftpVerbose, "psftp: Using userhost passed on commandline: %s", userhost);
 	int ret;
 	ret = psftp_connect(userhost, user, portnumber);
 	sfree(userhost);
