@@ -14,6 +14,7 @@ typedef struct
 {
 	std::string data;
 	CDirentry reference;
+	enum ServerType serverType;
 } t_entry;
 
 class CDirectoryListingParserTest : public CppUnit::TestFixture
@@ -31,6 +32,7 @@ public:
 
 	void testIndividual();
 	void testAll();
+	void testSpecial();
 
 	static std::vector<t_entry> m_entries;
 
@@ -56,7 +58,7 @@ static int calcYear(int month, int day)
 	// Not exact but good enough for our purpose
 	const int day_of_year = month * 31 + day;
 	const int cur_day_of_year = cur_month * 31 + cur_day;
-	if (day_of_year > cur_day_of_year)
+	if (day_of_year > (cur_day_of_year + 1))
 		return cur_year - 1;
 	else
 		return cur_year;
@@ -82,7 +84,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(8, wxDateTime::Apr, 1994),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// This one is a recent file with a time instead of the year.
@@ -100,7 +103,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(29, wxDateTime::Mar, calcYear(3, 29), 3, 26),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Group omitted
@@ -118,7 +122,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(8, wxDateTime::Apr, 1994),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Symbolic link
@@ -136,7 +141,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(25, wxDateTime::Jan, calcYear(1, 25), 0, 17),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Some listings with uncommon date/time format
@@ -156,7 +162,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(26, wxDateTime::Sep, 2000),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -173,7 +180,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(26, wxDateTime::Sep, calcYear(9, 26), 13, 45),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -190,7 +198,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(7, wxDateTime::Jun, 2005, 21, 22),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 
@@ -209,7 +218,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(5, wxDateTime::Oct, calcYear(10, 5), 21, 22),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// NetWare style listings
@@ -229,7 +239,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(16, wxDateTime::Jan, calcYear(1, 16), 18, 53),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -246,7 +257,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(20, wxDateTime::Oct, calcYear(10, 20), 15, 27),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// NetPresenz for the Mac
@@ -270,7 +282,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(22, wxDateTime::Nov, 1995),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -287,7 +300,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(10, wxDateTime::May, 1996),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// A format with domain field some windows servers send
@@ -305,7 +319,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(29, wxDateTime::Jan, calcYear(1, 29), 3, 26),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// EPLF directory listings
@@ -313,6 +328,8 @@ void CDirectoryListingParserTest::InitEntries()
 
 	// See http://cr.yp.to/ftp/list/eplf.html (mirrored at http://filezilla-project.org/specs/eplf.html)
 
+	wxDateTime utc(1, wxDateTime::Mar, 1996, 22, 15, 3);
+	utc.MakeFromTimezone(wxDateTime::UTC);
 	m_entries.push_back((t_entry){
 			"+i8388621.48594,m825718503,r,s280,up755\t14-eplf file",
 			{
@@ -325,11 +342,14 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(1, wxDateTime::Mar, 1996, 23, 15, 3),
+				utc,
 				false
-			}
+			},
+			DEFAULT
 		});
 
+	utc = wxDateTime(13, wxDateTime::Feb, 1996, 23, 58, 27);
+	utc.MakeFromTimezone(wxDateTime::UTC);
 	m_entries.push_back((t_entry){
         	"+i8388621.50690,m824255907,/,\t15-eplf dir",
 			{
@@ -342,9 +362,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(14, wxDateTime::Feb, 1996, 0, 58, 27),
+				utc,
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// MSDOS type listing used by old IIS
@@ -364,7 +385,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(27, wxDateTime::Apr, 2000, 12, 9),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Ambiguous date and AM/PM crap. Some evil manager must have forced the poor devs to implement this
@@ -382,7 +404,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(6, wxDateTime::Apr, 2000, 15, 47),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -399,7 +422,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(2, wxDateTime::Sep, 2002, 18, 48),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -416,10 +440,13 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(2, wxDateTime::Sep, 2002, 19, 6),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Numerical unix style listing
+	utc = wxDateTime(29, wxDateTime::Nov, 1973, 21, 33, 9);
+	utc.MakeFromTimezone(wxDateTime::UTC);
 	m_entries.push_back((t_entry){
 			"0100644   500  101   12345    123456789       20-unix-numerical file",
 			{
@@ -432,9 +459,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(29, wxDateTime::Nov, 1973, 22, 33, 9),
+				utc,
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// VShell servers
@@ -454,7 +482,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(4, wxDateTime::Apr, 2000, 21, 6),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -471,7 +500,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(12, wxDateTime::Dec, 2002, 2, 13),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* This type of directory listings is sent by some newer versions of VShell
@@ -490,7 +520,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(8, wxDateTime::Oct, 2002, 9, 47),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// OS/2 server format
@@ -511,7 +542,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(23, wxDateTime::Apr, 2003, 10, 57),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -528,7 +560,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(14, wxDateTime::Jul, 1999, 12, 37),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Another server not aware of Y2K
@@ -546,7 +579,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(11, wxDateTime::Feb, 2003, 16, 15),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Again Y2K
@@ -564,7 +598,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(5, wxDateTime::Oct, 2000, 23, 38),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Localized date formats
@@ -584,7 +619,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(26, wxDateTime::Jul, calcYear(7, 26), 20, 10),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -601,7 +637,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(2, wxDateTime::Oct, 2003),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -618,7 +655,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(12, wxDateTime::Oct, 1999, 17, 12),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -635,7 +673,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(24, wxDateTime::Apr, 2003, 17, 12),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Japanese listing
@@ -655,7 +694,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(18, wxDateTime::Apr, 2003),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Some other asian listing format. Those >127 chars are just examples
@@ -674,7 +714,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(20, wxDateTime::Mar, 2003),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -691,7 +732,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(17, wxDateTime::Aug, calcYear(8, 17), 8, 52),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -708,7 +750,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(15, wxDateTime::Jul, 2004),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// VMS listings
@@ -717,8 +760,8 @@ void CDirectoryListingParserTest::InitEntries()
 	m_entries.push_back((t_entry){
 			"36-vms-dir.DIR;1  1 19-NOV-2001 21:41 [root,root] (RWE,RWE,RE,RE)",
 			{
-				_T("36-vms-dir;1"),
-				1,
+				_T("36-vms-dir"),
+				512,
 				_T("RWE,RWE,RE,RE"),
 				_T("root,root"),
 				true,
@@ -728,7 +771,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(19, wxDateTime::Nov, 2001, 21, 41),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 
@@ -736,7 +780,7 @@ void CDirectoryListingParserTest::InitEntries()
 			"37-vms-file;1       155   2-JUL-2003 10:30:13.64",
 			{
 				_T("37-vms-file;1"),
-				155,
+				79360,
 				_T(""),
 				_T(""),
 				false,
@@ -744,9 +788,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30),
+				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 13),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* VMS style listing without time */
@@ -754,7 +799,7 @@ void CDirectoryListingParserTest::InitEntries()
 			"38-vms-notime-file;1    2/8    7-JAN-2000    [IV2_XXX]   (RWED,RWED,RE,)",
 			{
 				_T("38-vms-notime-file;1"),
-				2,
+				1024,
 				_T("RWED,RWED,RE,"),
 				_T("IV2_XXX"),
 				false,
@@ -764,7 +809,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(7, wxDateTime::Jan, 2000),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* Localized month */
@@ -772,7 +818,7 @@ void CDirectoryListingParserTest::InitEntries()
 			"39-vms-notime-file;1    6/8    15-JUI-2002    PRONAS   (RWED,RWED,RE,)",
 			{
 				_T("39-vms-notime-file;1"),
-				6,
+				3072,
 				_T("RWED,RWED,RE,"),
 				_T("PRONAS"),
 				false,
@@ -782,14 +828,15 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(15, wxDateTime::Jul, 2002),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
 			"40-vms-multiline-file;1\r\n170774/170775     24-APR-2003 08:16:15  [FTP_CLIENT,SCOT]      (RWED,RWED,RE,)",
 			{
 				_T("40-vms-multiline-file;1"),
-				170774,
+				87436288,
 				_T("RWED,RWED,RE,"),
 				_T("FTP_CLIENT,SCOT"),
 				false,
@@ -797,16 +844,17 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(24, wxDateTime::Apr, 2003, 8, 16),
+				wxDateTime(24, wxDateTime::Apr, 2003, 8, 16, 15),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
 			"41-vms-multiline-file;1\r\n10     2-JUL-2003 10:30:08.59  [FTP_CLIENT,SCOT]      (RWED,RWED,RE,)",
 			{
 				_T("41-vms-multiline-file;1"),
-				10,
+				5120,
 				_T("RWED,RWED,RE,"),
 				_T("FTP_CLIENT,SCOT"),
 				false,
@@ -814,9 +862,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30),
+				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 8),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// VMS style listings with a different field order
@@ -824,7 +873,7 @@ void CDirectoryListingParserTest::InitEntries()
 			"42-vms-alternate-field-order-file;1   [SUMMARY]    1/3     2-AUG-2006 13:05  (RWE,RWE,RE,)",
 			{
 				_T("42-vms-alternate-field-order-file;1"),
-				1,
+				512,
 				_T("RWE,RWE,RE,"),
 				_T("SUMMARY"),
 				false,
@@ -834,14 +883,15 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(2, wxDateTime::Aug, 2006, 13, 5),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
 			"43-vms-alternate-field-order-file;1       17-JUN-1994 17:25:37     6308/13     (RWED,RWED,R,)",
 			{
 				_T("43-vms-alternate-field-order-file;1"),
-				6308,
+				3229696,
 				_T("RWED,RWED,R,"),
 				_T(""),
 				false,
@@ -849,9 +899,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(17, wxDateTime::Jun, 1994, 17, 25),
+				wxDateTime(17, wxDateTime::Jun, 1994, 17, 25, 37),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	// Miscellaneous listings
@@ -870,9 +921,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9),
+				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -887,9 +939,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9),
+				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* aligned directory listing with too long size */
@@ -907,7 +960,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(12, wxDateTime::Feb, calcYear(2, 12), 17, 20),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* short directory listing with month name */
@@ -925,7 +979,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(1, wxDateTime::Jun, 1999),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* Nortel wfFtp router */
@@ -941,9 +996,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(3, wxDateTime::Jun, 2004, 10, 20),
+				wxDateTime(3, wxDateTime::Jun, 2004, 10, 20, 3),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* VxWorks based server used in Nortel routers */
@@ -959,9 +1015,10 @@ void CDirectoryListingParserTest::InitEntries()
 				_T(""),
 				true,
 				true,
-				wxDateTime(28, wxDateTime::Feb, 1998, 5, 23),
+				wxDateTime(28, wxDateTime::Feb, 1998, 5, 23, 30),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* the following format is sent by the Connect:Enterprise server by Sterling Commerce */
@@ -979,7 +1036,8 @@ void CDirectoryListingParserTest::InitEntries()
 				true,
 				wxDateTime(13, wxDateTime::Jan, calcYear(1, 13), 7, 42),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* Microware OS-9
@@ -998,7 +1056,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(29, wxDateTime::Mar, 2007),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	/* Localised Unix style listing. Month and day fields are swapped */
@@ -1016,7 +1075,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(1, wxDateTime::Oct, 2004),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -1033,7 +1093,8 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(11, wxDateTime::Nov, 2005),
 				false
-			}
+			},
+			DEFAULT
 		});
 
 	m_entries.push_back((t_entry){
@@ -1051,7 +1112,103 @@ void CDirectoryListingParserTest::InitEntries()
 				false,
 				wxDateTime(6, wxDateTime::Jan, 2005),
 				false
-			}
+			},
+			DEFAULT
+		});
+
+	/* IBM AS/400 style listing with localized date*/
+	m_entries.push_back((t_entry){
+			"QPGMR           36864 18.09.06 14:21:26 *FILE      55-AS400.FILE",
+			{
+				_T("55-AS400.FILE"),
+				36864,
+				_T(""),
+				_T("QPGMR"),
+				false,
+				false,
+				_T(""),
+				true,
+				true,
+				wxDateTime(18, wxDateTime::Sep, 2006, 14, 21, 26),
+				false
+			},
+			DEFAULT
+		});
+
+	/* VMS style listing with complex size */
+	m_entries.push_back((t_entry){
+			"56-VMS-complex-size;1 2KB 23-SEP-2005 14:57:07.27",
+			{
+				_T("56-VMS-complex-size;1"),
+				2048,
+				_T(""),
+				_T(""),
+				false,
+				false,
+				_T(""),
+				true,
+				true,
+				wxDateTime(23, wxDateTime::Sep, 2005, 14, 57, 07),
+				false
+			},
+			DEFAULT
+		});
+
+	/* HP NonStop */
+	m_entries.push_back((t_entry){
+			"57-HP_NonStop 101 528 6-Apr-07 14:21:18 255, 0 \"oooo\"",
+			{
+				_T("57-HP_NonStop"),
+				528,
+				_T("\"oooo\""),
+				_T("255, 0"),
+				false,
+				false,
+				_T(""),
+				true,
+				true,
+				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18),
+				false
+			},
+			HPNONSTOP
+		});
+
+	// Only difference is in the owner/group field, no delimiting space.
+	m_entries.push_back((t_entry){
+			"58-HP_NonStop 101 528 6-Apr-07 14:21:18 255,255 \"oooo\"",
+			{
+				_T("58-HP_NonStop"),
+				528,
+				_T("\"oooo\""),
+				_T("255,255"),
+				false,
+				false,
+				_T(""),
+				true,
+				true,
+				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18),
+				false
+			},
+			HPNONSTOP
+		});
+
+
+	m_entries.push_back((t_entry){
+			"drwxr-xr-x 6 user sys 1024 30. Jan., 12:40 59-localized-date-dir",
+			{
+				_T("59-localized-date-dir"),
+				1024,
+				_T("drwxr-xr-x"),
+				_T("user sys"),
+				true,
+				false,
+				_T(""),
+				true,
+				true,
+				wxDateTime(30, wxDateTime::Jan, calcYear(1, 30), 12, 40),
+				false
+			},
+			DEFAULT
 		});
 
 /*
@@ -1084,7 +1241,8 @@ void CDirectoryListingParserTest::testIndividual()
 
 	m_sync.Leave();
 
-	const CServer server;
+	CServer server;
+	server.SetType(entry.serverType);
 
 	CDirectoryListingParser parser(0, server);
 
@@ -1109,10 +1267,12 @@ void CDirectoryListingParserTest::testIndividual()
 
 void CDirectoryListingParserTest::testAll()
 {
-	const CServer server;
+	CServer server;
 	CDirectoryListingParser parser(0, server);
 	for (std::vector<t_entry>::const_iterator iter = m_entries.begin(); iter != m_entries.end(); iter++)
 	{
+		server.SetType(iter->serverType);
+		parser.SetServer(server);
 		const char* str = iter->data.c_str();
 		const int len = strlen(str);
 		char* data = new char[len];
